# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2021/6/28 22:11
describe: 使用掘金的数据，查看任意时刻的标的状态，精确到分钟
"""

from src.utils.bt import GmCzscTrader
from czsc.signals import get_default_signals


if __name__ == '__main__':
    get_signals = get_default_signals

    # ct = GmCzscTrader("SZSE.300511", end_dt='2021-07-15 14:38:00+08:00', get_signals=get_signals)
    ct = GmCzscTrader("SZSE.300511", get_signals=get_signals)
    ct.open_in_browser()




# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2021/10/24 16:12
describe: Tushare 数据缓存
"""
import shutil

from .ts import *
from ..utils import io


class TsDataCache:
    """Tushare 数据缓存"""
    def __init__(self, data_path, sdt, edt, verbose=False):
        """

        :param data_path: 数据路径
        :param sdt: 缓存开始时间
        :param edt: 缓存结束时间
        :param verbose: 是否显示详细信息
        """
        self.date_fmt = "%Y%m%d"
        self.verbose = verbose
        self.sdt = pd.to_datetime(sdt).strftime(self.date_fmt)
        self.edt = pd.to_datetime(edt).strftime(self.date_fmt)
        self.data_path = data_path
        self.prefix = "TS_CACHE"
        self.name = f"{self.prefix}_{self.sdt}_{self.edt}"
        self.cache_path = os.path.join(self.data_path, self.name)
        os.makedirs(self.cache_path, exist_ok=True)

        self.freq_map = {
            "D": Freq.D,
            "W": Freq.W,
            "M": Freq.M,
        }

    def clear(self):
        """清空缓存"""
        for path in os.listdir(self.data_path):
            if path.startswith(self.prefix):
                path = os.path.join(self.data_path, path)
                shutil.rmtree(path)
                if self.verbose:
                    print(f"clear: remove {path}")
                if os.path.exists(path):
                    print(f"Tushare 数据缓存清理失败，请手动删除缓存文件夹：{self.cache_path}")

    # ------------------------------------Tushare 原生接口----------------------------------------------
    def ths_daily(self, ts_code, start_date, end_date, raw_bar=True):
        """获取同花顺概念板块的日线行情"""
        file_cache = os.path.join(self.cache_path, f"{ts_code}.pkl")
        if os.path.exists(file_cache):
            kline = io.read_pkl(file_cache)
            if self.verbose:
                print(f"ths_daily: read cache {file_cache}")
        else:
            kline = pro.ths_daily(ts_code=ts_code, start_date=self.sdt, end_date=self.edt,
                                  fields='ts_code,trade_date,open,close,high,low,vol')
            kline = kline.sort_values('trade_date', ignore_index=True)

            for bar_number in (1, 2, 3, 5, 10, 20):
                n_col_name = 'n' + str(bar_number) + 'b'
                kline[n_col_name] = (kline['close'].shift(-bar_number) / kline['close'] - 1) * 10000

            io.save_pkl(kline, file_cache)

        kline['trade_date'] = pd.to_datetime(kline['trade_date'], format=self.date_fmt)
        start_date = pd.to_datetime(start_date)
        end_date = pd.to_datetime(end_date)
        bars = kline[(kline['trade_date'] >= start_date) & (kline['trade_date'] <= end_date)]
        if raw_bar:
            bars = format_kline(bars, freq=Freq.D)
        return bars

    def ths_index(self, exchange="A"):
        """获取同花顺概念

        https://tushare.pro/document/2?doc_id=259
        :param exchange:
        :return:
        """
        file_cache = os.path.join(self.cache_path, f"ths_index_{exchange}.pkl")
        if os.path.exists(file_cache):
            df = io.read_pkl(file_cache)
            if self.verbose:
                print(f"ths_index: read cache {file_cache}")
        else:
            df = pro.ths_index(exchange=exchange)
            io.save_pkl(df, file_cache)
        return df

    def ths_member(self, ts_code):
        """获取同花顺概念成分股

        https://tushare.pro/document/2?doc_id=261
        :param ts_code:
        :return:
        """
        file_cache = os.path.join(self.cache_path, f"ths_members_{ts_code}.pkl")
        if os.path.exists(file_cache):
            df = io.read_pkl(file_cache)
        else:
            df = pro.ths_member(ts_code=ts_code,
                                fields="ts_code,code,name,weight,in_date,out_date,is_new")
            io.save_pkl(df, file_cache)
        return df

    def pro_bar(self, ts_code, start_date, end_date, freq='D', asset="E", raw_bar=True):
        """获取日线以上数据

        https://tushare.pro/document/2?doc_id=109

        :param ts_code:
        :param start_date:
        :param end_date:
        :param freq:
        :param asset: 资产类别：E股票 I沪深指数 C数字货币 FT期货 FD基金 O期权 CB可转债（v1.2.39），默认E
        :param raw_bar:
        :return:
        """
        file_cache = os.path.join(self.cache_path, f"{ts_code}_{asset}_{freq}.pkl")
        if os.path.exists(file_cache):
            kline = io.read_pkl(file_cache)
            if self.verbose:
                print(f"pro_bar: read cache {file_cache}")
        else:
            kline = ts.pro_bar(ts_code=ts_code, asset=asset, adj='qfq', freq=freq,
                               start_date=self.sdt, end_date=self.edt)
            kline = kline.sort_values('trade_date', ignore_index=True)
            for bar_number in (1, 2, 3, 5, 10, 20):
                n_col_name = 'n' + str(bar_number) + 'b'
                kline[n_col_name] = (kline['close'].shift(-bar_number) / kline['close'] - 1) * 10000

            io.save_pkl(kline, file_cache)

        kline['trade_date'] = pd.to_datetime(kline['trade_date'], format=self.date_fmt)
        start_date = pd.to_datetime(start_date)
        end_date = pd.to_datetime(end_date)
        bars = kline[(kline['trade_date'] >= start_date) & (kline['trade_date'] <= end_date)]
        if raw_bar:
            bars = format_kline(bars, freq=self.freq_map[freq])
        return bars

    def index_classify(self, level="L1"):
        """
        https://tushare.pro/document/2?doc_id=181

        :param level:
        :return:
        """
        file_cache = os.path.join(self.cache_path, f"sw_index_{level}.pkl")
        if os.path.exists(file_cache):
            df = io.read_pkl(file_cache)
        else:
            df = pro.index_classify(level=level, src='SW')
            io.save_pkl(df, file_cache)
        return df

    def index_member(self, index_code):
        """
        https://tushare.pro/document/2?doc_id=182

        :param index_code:
        :return:
        """
        file_cache = os.path.join(self.cache_path, f"sw_members_{index_code}.pkl")
        if os.path.exists(file_cache):
            df = io.read_pkl(file_cache)
        else:
            df = pro.index_member(index_code=index_code)
            io.save_pkl(df, file_cache)
        return df

    def stock_basic(self):
        """
        https://tushare.pro/document/2?doc_id=25

        :return:
        """
        file_cache = os.path.join(self.cache_path, f"stock_basic.pkl")
        if os.path.exists(file_cache):
            df = io.read_pkl(file_cache)
        else:
            df = pro.stock_basic(exchange='', list_status='L',
                                 fields='ts_code,symbol,name,area,industry,list_date')
            io.save_pkl(df, file_cache)
        return df

    # ------------------------------------CZSC 加工接口----------------------------------------------

    def get_all_ths_members(self):
        """获取同花顺A股全部概念列表"""
        file_cache = os.path.join(self.cache_path, "all_ths_members.pkl")
        if os.path.exists(file_cache):
            df = io.read_pkl(file_cache)
        else:
            concepts = self.ths_index(exchange='A')
            concepts = concepts.to_dict('records')

            res = []
            for concept in concepts:
                _df = self.ths_member(ts_code=concept['ts_code'])
                _df['概念名称'] = concept['name']
                _df['概念代码'] = concept['ts_code']
                res.append(_df)
                time.sleep(0.3)

            df = pd.concat(res, ignore_index=True)
            io.save_pkl(df, file_cache)
        return df




# coding: utf-8

from enum import Enum


class Operate(Enum):
    # 持有状态
    HL = "持多"  # Hold Long
    HS = "持空"  # Hold Short
    HO = "持币"  # Hold Other

    # 多头操作
    LO = "开多"          # Long Open
    LE = "平多"          # Long Exit
    LA1 = "第一次加多仓"  # Long Add 1
    LA2 = "第二次加多仓"  # Long Add 2
    LR1 = "第一次减多仓"  # Long Reduce 1
    LR2 = "第二次减多仓"  # Long Reduce 2

    # 空头操作
    SO = "开空"          # Short Open
    SE = "平空"          # Short Exit
    SA1 = "第一次加空仓"  # Short Add 1
    SA2 = "第二次加空仓"  # Short Add 2
    SR1 = "第一次减空仓"  # Short Reduce 1
    SR2 = "第二次减空仓"  # Short Reduce 2


class Mark(Enum):
    D = "底分型"
    G = "顶分型"


class Direction(Enum):
    Up = "向上"
    Down = "向下"


class Freq(Enum):
    Tick = "Tick"
    F1 = "1分钟"
    F5 = "5分钟"
    F15 = "15分钟"
    F30 = "30分钟"
    F60 = "60分钟"
    D = "日线"
    W = "周线"
    M = "月线"
    S = "季线"
    Y = "年线"



# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2021/9/25 13:38
describe: 摘录一些缠中说禅良言警句，希望能帮助更多人走出来。
"""
import random

texts = {
    "1": """
一定要注意，你不是多头也不是空头，但你一定要知道一旦发生什么情况，多头会干什么，空头会
干什么。例如，多头的愿望肯定是想补上面的缺口，这就构成市场的一个潜在力量，这个力量，在
其他力量的干扰下，可能一时发挥不出来，但这反而构成我们一个更好的买入点。请好好品味这句
话：你不当多头也不当空头，但一定要知道空头多头想干什么，而走势是最终的结果，他们想干的
是否干出来了，这才是关键。干不出来，有什么后果，他们会有什么后续的步骤，这才是该想的东
西。

--摘自《2008-01-29 15:19 年线支持初显现》""",

    "2": """
要战胜市场，首先要了解市场的众生。市场是合力的，而合这力的不是机械，而是活生生的人。市
场中，最多数的，都是糊涂蛋，赚钱了不知道为什么，亏钱了不知道为什么，最后变青蛙了，也会
说，井上面的天空好大，好复杂，怎么处理啊？哪里有拐杖啊？几乎绝大多数的人，进市场来时，
根本不知道市场是什么，然后就不断投入，最后有些输红眼了，砸锅卖铁也就进来了。对于市场，
本ID有一个观点，大概有点过分，但确实是对的。市场，就是要0投入去赚钱。

--摘自《2008-01-22 16:10 教你炒股票95：修炼自己》""",

    "3": """
要认清市场，首先要认清自己，知道自己的弱点在哪里，自己在市场中的每个行为，都要清楚地意
识到。每天收盘后，都找十分钟，把自己当天的操作以及看盘时的心理过程复一次盘，这是十分必
要的。

--摘自《2007-04-04 15:31 教你炒股票42：有些人是不适合参与市场的》""",

    "4": """
今天该股90%多的换手意味着，所有中签的人基本都抛了（对手盘包括其中），而且绝大多数都在
10几元抛的。这就对了，你们10几元不看好，有人要看好，市场向所有人开着，凭什么不能买？
市场就是一个斗心理的过程，大家都想一块去了，大家还用不用活了？

--摘自《2006-06-19 16:45 鄙视所有对N中工15元不敢买50元就吃醋的人！》""",

    "5": """
你的喜好，就是你的死亡陷阱！在市场中要生存，第一条就是在市场中要杜绝一切喜好。市场的诱
惑，永远就是通过你的喜好而陷你于死亡。市场中需要的是露水之缘而不是比翼之情，天长地久之
类的东西和市场无关。市场中唯一值得天长地久的就是赢钱，任何一个来市场的人，其目的就是赢
钱，任何与赢钱无关的都是废话。必须明白，任何让你买入一只股票的理由，并不是因为这股票如
何好或被忽悠得如何好，只是你企图通过买入而赢钱，能赢钱的股票就是好的，否则都是废话。因
此，市场中的任何喜好，都是把你引入迷途的陷阱，必须逐一看破，进而洗心革面，才能在市场上
生存。

--摘自《2006-06-09 17:03 教你炒股票3：你的喜好，你的死亡陷阱！》""",

    "6": """
能看清楚自己周围的市场陷阱，还只是第一步，更进一步，要学会利用市场陷阱来赢钱。当你要买
的时候，空头的陷阱就是你的最佳机会，当你要卖的时候，多头的陷阱当然就是你的天堂。这市场
，永远不缺卖在最低点，买在最高点的人，这世界上没有什么是可以让所有人赢钱的，连大牛市中
都有很多人要亏损累累。而市场中的行为，就如同一个修炼上乘武功的过程，最终能否成功，还是
要落实到每个人的智慧、秉性、天赋、勤奋上来！

--摘自《2006-06-09 17:03 教你炒股票3：你的喜好，你的死亡陷阱！》""",

    "7": """
市场是有规律的，但市场的规律并不是显而易见的，是需要严格的分析才能得到。更重要的是，市
场的规律是一种动态的，在不同级别合力作用下显示出来的规律，企图用些单纯的指标、波段、 
波浪、分型、周期等等预测、把握，只可能错陋百出。但只要把这动态的规律在当下的直观中把握
好、应用纯熟，踏准市场的节奏，并不是不可能的。

--摘自《2006-12-27 15:18 教你炒股票19：学习缠中说禅技术分析理论的关键》""",

    "8": """
战胜市场，其实就是战胜自己的贪婪、恐惧、愚蠢，本ID的理论只是把市场拔光给各位看，而拔光
一个人并不意味着就等于征服一个人，对于市场，其道理是一样的。不干，不可能征服市场。对于
市场来说，干就是一切。技术分析的最终意义不是去预测市场要干什么，而是市场正在干什么，是
一种当下的直观。在市场上所有的错误都是离开了这当下的直观，用想象、用情绪来代替。

--摘自《2006-12-27 15:18 教你炒股票19：学习缠中说禅技术分析理论的关键》""",

    "9": """
熟悉本ID所解《论语》的都知道，风险是“不患”的，是无位次的，任何妄求在投资中的绝对无风险，
都是痴心妄想。唯一的办法，就是设置一个系统，使得无位次、“不患”的风险在该系统中成为有位
次，“患”的系统，这是长期战胜市场的唯一方法。

--摘自《2006-12-01 12:03 教你炒股票12：一吻何能消魂？》""",

    "10": """
贪婪和恐惧，人的死穴，周末到了，请给自己一小时去找找自己！

--摘自《2007-01-26 15:03 罗杰斯，有种的和本ID来个PK》
""",

    "11": """
任何个人、阶级，其力量归根结底来自自己，无须期盼或感恩于所谓的救世主、大救星。所有救世
主、大救星都不过是人造的，连神都是人造的，没有造的人，即使是神，也什么都不是！    

--摘自《2006-04-23 18:32 鼓吹救世主、大救星的是真正的精神鸦片！》
""",

    "12": """
每一个人，都是天地不能覆盖，世界的风云雷电就在每个人脚下，自我憋屈只不过是去成就如大救
星般的无聊玩意。历史由每个人所创造，历史由每个人而命名，如果真有什么大救星，你就是你的
大救星！

--摘自《2006-04-19 21:42 布什，当今世界的大救星！》
""",

    "13": """
市场中没有人会同情你，当你哀求着股市跌时不要跌，楼市涨时不要涨，最终都不过成为了市场中
的炮灰。市场只有在所有能成为炮灰的都成为炮灰后，才会休息或改变方向。市场就是市场，市场
上的炮灰都是那些企图成为“大救星”或企图成为被“大救星”的人，而人类社会这个大市场也一样！

--摘自《2006-04-24 21:08 中国楼市股市的闹剧，都是“大救星”思维流毒所致！》
""",

    "14": """
操作其实很简单，一个基本的原则就是，任何走势，无论怎么折腾，都逃不出这个节奏，就是底、
顶以及连接两者的中间过程，因此，在两头的操作节奏就是中枢震荡，只是底的时候要先买后卖，
顶的时候要先卖后买，这样更安全点。至于中间的连接部分，就是持有，当然，对于空头走势，小
板凳就是一个最好的持有，一直持有到底部构造完成。而有技术的，根本就不需要什么小板凳，按
操作级别，分清楚目前是三阶段中的哪一段，然后日日是好日，时时是花时，不赚钱那真是脑子有
水了。亏钱都是错误操作引起的，不断反省，才会有进步的。

--摘自《2008-08-29 09:15 教你炒股票108：何谓底部？从月线看中期走势演化》
""",

    "15": """
底部都是分级别的，如果站在精确走势类型的角度，那么第一类买点出现后，直到该买点所引发的
中枢第一次走出第三类买卖点前，都可以看成底部构造的过程。只不过如果是第三类卖点先出现，
就意味着这底部构造失败了，反之，第三类买点意味着底部构造的最终完成并展开新的行情。当然
，顶部的情况，反过来定义就是。有了这个定义，就一定要搞明白，不是在底部的区间上买，而是
相反，应该和中枢震荡的操作一样，在区间下探失败时买，这才是最好的买点。

--摘自《2008-08-29 09:15 教你炒股票108：何谓底部？从月线看中期走势演化》
""",

    "16": """
牛人，一般是指站在潮流之巅的。在投资市场里，整体的失败是一次都不能发生的，只要发生一次
，基本就翻不了身了。个别的失败是允许的，但不能影响大局。

最早时几千万就可以当庄家了，现在几千万连大散户都算不上，在投资市场中，一次的跌倒，终生
都追不回来，基本只能在后面跟着玩了。而在后面跟着玩，怎么都算不了牛人。

--摘自《2006-11-20 12:00 教你炒股票8：投资如选面首，G点为中心，拒绝ED男！》评论
""",

    "17": """
意画心描自主奴，不管你相信与否，你的命运都是自己造成的，你的命运的改变也是自己造成的，
没有你自己种下的种子，佛也救不了你。有空，就好好看看你自己这个世界吧，看看自己曾种下什
么，又有什么正萌发，又有什么还在潜伏，又有什么被新种下了。善种子、恶种子，堆满你的世界。

--摘自《2008-01-20 21:01 刚到家，说两句》
""",

    "18": """
市场就如同一头牛，只有目无全牛，才可能随心解之而合其关节。在本ID的理论中，机械化操作
的本质就是目无全牛而合其关节，因为，根据本ID的理论，市场的结构已经被彻底分解，站在本
ID理论的角度，哪里有什么市场，不过是一堆的关节。而机械化操作，就是逐步合于其关节的节
奏，而不被全牛的繁复所影响。

--摘自《2008-04-13 21:51 教你炒股票105：远离聪明、机械操作》
""",

    "19": """
挣钱，本来就是很简单的事情，不过就是一个良好习惯与操作策略的结果，一点劲都不用费。那些
费力才能挣到的钱，也不会袋得住。

--摘自《2008-04-13 21:51 教你炒股票105：远离聪明、机械操作》
""",

    "20": """
股票这种面首，一定要控制成本，不要追高，有技术的，一定要通过震荡把成本往0甚至负处玩弄
下去，这才是玩弄股票之道。

--摘自《2007-07-09 15:35 中国股市前途的大决战》
""",

    "21": """
不要用你的想象代替现实。股市里的牛人每年都有，死去的牛人更多，市场的第一原则就是生存，
只要你30年后还能活下来，自然就是最大的牛人。

--摘自《2006-11-20 12:00 教你炒股票8：投资如选面首，G点为中心，拒绝ED男！》评论
""",

    "22": """
要长期胜利，就一定要坚持用最小风险换取最大利润，风险是第一的，这里没有什么高低之分。亏
损是按百分比的，一百亿和一百万，亏了百分百，都是零。人弃我不一定取，人抢我一定给。

--摘自《2006-11-16 12:00 教你炒股票7：给赚了指数亏了钱的一些忠告》评论
""",

    "23": """
有人说每天看盘很累很疲惫，关键是对市场没有充分理解，看盘何尝不是一种放松，不能放松，是
因为有烦恼，被烦恼所烦恼，智慧才是最大的放松。
 
--摘自《即缠非缠》微博
""",

    "24": """
在现实中，所谓的天命，都是在人谋之中，只是你的谋划是否完全，另外，一个很重要的是，完全
的谋划是否超越你的能力。

对于股票来说，完全的分类或谋划，基本不存在超越能力的问题，只是买卖多少的问题，有能力就
多点，没能力就少点，不存在某种分类完全不能执行的情况。因此，所有的重点，都在这完全的分
类上了。但完全的分类，不是单层次的，一定也必须是多层次的。本ID的理论最重要的特点之一，
就是自然给出了分类的层次，也就是不同的自然形成的级别。不同的级别，有不同的完全分类，而
综合起来，就有了一个立体的完全分类的系统，这才是我们的操作必须依赖的。

把市场分析好了，把情况分类好了，然后问一下自己，你有这个处理所有可能情况的能力吗？如果
没有，那就算了；如果有，就上。事情就这么简单。

--摘自《2008-02-25 16:32 教你炒股票100：中医、兵法、诗歌、操作3》
""",

    "25": """
小人，虽说是对利益之网而游刃有余，但因为画地为牢，自我小之，为一我所牵，最终不过仍是机
关木人，到头来，被一点聪明所误，一场游戏一场梦而已。而君子，尽知小人所知，行小人之行而
无一我之所牵，不废一法而行千法万法，行千法万法而不立一法，于所知所行而自在。

--摘自《2007-08-20 22:36 《论语》详解：给所有曲解孔子的人（67）》
""",

    "26": """
人，总是自己骗自己；人，只能自己骗自己。没有你自己，谁都骗不到你。

--摘自《2008-01-27 10:15 教你打坐22：佛魔最难除》
""",

    "27": """
你在操作时，你后续的所有可能面对的情况与对策都必须了然，否则你就没资格操作。对于一个真
正的操作者，没有任何情况是意外的，因为，所有的情况都被完全地分类了，所有相应的对策都事
先有了，只是等着市场自己去选择，去触及我们事先给定的开关。

--摘自《2008-02-04 19:51 教你炒股票98：中医、兵法、诗歌、操作2》
""",

    "28": """
市场中，最大的敌人之一，就赌徒心理、赌徒思维。赌，最终的结局就只有一个，如果你以赌徒心
理参与市场，那么你的结局就已经注定，你就算还没再锅里，那也只是养肥了再煮而已，没什么区
别。

赌徒心理一个最大的特点，就是预设一个虚拟的目标，一个想象中的目标，完全无视市场本身。

--摘自《2008-01-23 16:18 教你炒股票96：无处不在的赌徒心理》
""",

    "29": """
市场真正的成功，都是严格的操作下完成的。操作失误了有什么大不了的，市场的机会不断涌现，
一个严格的操作程序，足以保证你长期的成功。

生活，很简单，一天三顿，五谷为养、五果为助、五畜为益、五菜为充，而不是那些古灵精怪的玩
意；市场很简单，就如同生活，在一定的韵律中生长出利润。只有那韵律，那平凡但又能长久的赢
利模式，才能使得你战胜市场。市场，只是生活的一部分，如此而已。

--摘自《2008-01-23 16:18 教你炒股票96：无处不在的赌徒心理》
""",

    "30": """
在本ID这里，只有严格分类后的不同操作类型，没有其他那么多无聊的不切边际的所谓预测。来本
ID这里学习，第一层次，就是要达到：当机立断。

机会，是可以预先分析的，但这分析，不是预测，而是建立在完全分类基础上的边界分划，这分划
完全来自本ID理论的纯数学构造，这构造的唯一性与精确性保证了这分类与边界的当下确认性。任
何一个当下，你都可以根据本ID的理论马上给出后面必然要出现的机会。

--摘自《2008-01-21 17:29 教你炒股票94：当机立断》
""",

    "31": """
学了本ID的理论，脑子里必须时刻有两个字：级别。如果连级别都搞不清楚，你还419？被419还
差不多。有了级别，就是节奏问题了，419，就是见好要收，而不是天长地久，这都不明白，就等
着灾难连连吧。不会卖出，就等于失去了下次买入的机会。这个节奏之所以难，说白了，就是贪嗔
痴疑慢作怪。

--摘自《2008-01-21 17:29 教你炒股票94：当机立断》
""",

    "32": """
投资是一门艺术，而投资的艺术归根结底是资金管理的艺术，这就像歌唱的艺术，归根结底是呼吸
的艺术一样。而市场的波动，归根结底是在前后两个高低点关系构成的一个完全分类中展开的，明
白了这一点，市场就如同自己的掌纹一样举手可见了。

--摘自《2006-05-12 19:02 股市闲谈：G股是G点，大牛不用套！》
""",

    "33": """
艺术高度往往和素材或技术无关，是否能把各种因素结合成一个生命的整体，才是艺术的关键。

--摘自《2006-02-20 10:04 连载1：“睡过”或“消费过”的男人们！》
""",

    "34": """
在市场中，第一就要分清楚预测与操作的严重区别：预测是游戏，是茶余饭后的谈资；而操作是真
刀真枪去干，是血与火的斗争。绝对的预测归根结底都是笑话，而非绝对的操作归根结底是死路一
条。像这次，绝对的操作是怎么样，本ID早已反复说明：3656点站不住走，3300缺口位置不回补
可以回补筹码再短差一把，否则就让大盘越南盾去。这里，整个操作的设计是没有任何不确定的地
方，都是绝对性的，这才能股票股票而不是被股票股票，而任何预测都只能永远在被股票股票的死
路上轮回。

--摘自《2008-06-10 16:40 今年诸多灾难与妖蛾子事的历史性解释》
""",

    "35": """
当前一段领涨个股和大盘走势相背离的时候，往往意味着一个结构性震荡的来临，这是股票中的常
见现象。

--摘自《2006-03-11 22:02 用股票技术分析的方法考察毛泽东现象！》
""",

    "36": """
股票就是废纸，该卖的时候不卖，把股票当宝，这就是投资的最大软肋。如果你看图形操作时，做
不到无我无股票，只有走势图形，那基本可以不看图了，因为有我有股票，被自己的贪婪恐惧所牵
引，你看的图，也不过就是自己的贪婪与恐惧，那何必看图？

请回想一下那些卖点时，你自己究竟在干什么？心里是不是有很多幻想，被幻想蒙蔽了眼睛？看图
操作，唯一的对象只有图，谁说都没用，市场是当下发生着的，没有人能替你去反应。

--摘自《2007-06-04 22:34 教你炒股票58：图解分析示范三》
""",

    "37": """
技术上，关键是看好各种底部形态的颈线位置的具体走势，这对短线发现好股票有帮助。

在震荡中，要注意千万别追高。

--摘自《2007-06-13 08:22 《论语》详解：给所有曲解孔子的人（66）》附录
""",

    "38": """
做股票，说白了就是忽悠着冲锋陷阱，只是你去忽悠别人，别让别人忽悠你。既然08年属于早收
割早有面包的年份，我们当然要在年初就大力忽悠。说实在，“向5600高地攻击前进”这点小目
标，说出来都不好意思，也太低了，不过先忽悠低的，现在的人胆子小，毒药要慢慢喂，不像以
前，说年内冲10000点，都有人和你急，他愣要说10000太低，还是12500比较好，50个250呀。

--摘自《2008-01-03 15:18 向5600高地攻击前进》附录
""",

    "39": """
走势反映的是人的贪嗔痴疑慢，如果你能通过走势当下的呈现，而观照其中参与的心理呈现，就等
于看穿了市场参与者的内心。心理，不是虚无飘渺的，最终必然要留下痕迹，也就是市场走势本身
。而一些具有自相似性的结构，就正好是窥测市场心理的科学仪器。

--摘自《2007-09-24 21:31 教你炒股票82：分型结构的心理因素》
""",

    "40": """
消息面、政策面、资金面，这面那面，最终作用的都是人心，人心因预期而交易，这里关系的就是
人的贪婪与恐惧、人的贪嗔痴疑慢。

--摘自《2007-06-19 08:04 教你炒股票60：图解分析示范五》
""",

    "41": """
不预测、不预期，并不是不可预测、不可预期，而是不为贪婪恐惧而预期、预测，是根据走势的自
身规律来。走势是有规律的，这规律是不患的，这不患的根源在于人贪嗔痴疑慢的不患。为什么本
ID要强调当下分解的多样性？因为走势本身就是当下形成中的，是市场各种预期的合力当下画出来
的，而每种画法都是不患的，都是源自人的贪嗔痴疑慢，因此每种多样性的分解都是符合理论的，
多样性不是模糊性，而是多角度去让市场本身自己去画地为牢，由此使得市场的走势万变不离本ID
理论的控制之中，而这，恰好是市场自身的规律之一。

--摘自《2007-06-19 08:04 教你炒股票60：图解分析示范五》
""",

    "42": """
市场上不是每一笔钱都适合任何人去赚的，面对市场的机会，少点贪嗔痴疑慢，认清自己的能力，
这比什么都重要。

市场是连续的，高位走了不是天堂，高位没走不是地狱。大跌，不过是下一买点后大反弹的前戏。
这一切，都逃不过本ID的理论，而是否参与，则与你的操作级别相关，也和你的操作能力相关。

--摘自《2007-09-11 21:38 教你炒股票80：市场没有同情、不信眼泪》
""",

    "43": """
没有人天生就是胜利者，也没有人天生就与失败为伍。人人是佛，无一人可度，无一人需救，人人
有明珠一颗，照破山河大地，又何必憋屈了自己？

--摘自《2007-09-11 21:38 教你炒股票80：市场没有同情、不信眼泪》
""",

    "44": """
对于散户来说，本质上没有卖错，只有买错。为什么？卖错又不会亏钱，买错就不同了。卖错了，
有钱，这么多股票可以被面首，为什么要一棵树吊死？

--摘自《2007-09-11 21:38 教你炒股票80：市场没有同情、不信眼泪》
""",


    "45": """
股票从来就不是股票，而是你的贪嗔痴疑慢；没有任何的失败相关于股票，而只关于你的贪嗔痴疑
慢，股票不过是一个幌子，一个道具。在西方，真正在资本市场上有成就的，基本都成了哲学家。
没有对市场的洞察，靠整天这消息、那题材地折腾，那永远只能在散户的区间中震荡。有此眼界，
不一定能达此高度，毕竟眼高手低也是通病；但无此眼界，就一定不可能达此高度。

--摘自《2007-09-11 21:38 教你炒股票80：市场没有同情、不信眼泪》
""",

    "46": """
万古长空，一朝风月，没有任何走势值得叨唠的，过去的就过去了，关键是后面的操作。无论前面
的操作是成功还是失败，只要市场在，人在，股票在，资金在，这游戏就继续运转，没有必然的胜
利者和失败者，关键是从此刻起，与自己的贪婪与恐惧说再见，深刻了解自己的能力与可能性，形
成一套适合自己的操作原则，这样，失败的可以胜利，胜利的继续胜利。

--摘自《2007-09-11 21:38 教你炒股票80：市场没有同情、不信眼泪》
""",

    "47": """
请注意中枢的递归定义的存在性意义，与分型、笔、线段的操作性意义的区别。

--摘自《2007-07-20 08:26 刚起来，下午解不了盘》
""",

    "48": """
大盘无分歧地创新高，才会导致真正的分歧。前天说了，现在最大的分歧就在于真假突破。但无论
真假突破，突破后的调整、回抽、确认等是必然要发生的。站在周线的角度，无非四种：一种是本
周的上影线调整；二是下周的周K线调整。前者是弱的，后者是强的。当然，还有一种更强的，就
是下周继续拉长阳，中间有日的跳水洗盘，再下周才真调整。最强的就是强力夹空，连收N根周阳，
全部调整以日线完成。

--摘自《2007-07-27 08:01 大盘周线上的四种演化分析》
""",

    "49": """
对于散户来说，一旦行情展开，就一定要按照行情自身的规律，喊口号不是操作，股票是用来操作
而不是用来喊口号的。如果大盘能保持基本在前期两高位连线上进行震荡整理，那么就将走强，否
则就要面对假突破后的猛烈清理。

--摘自《2007-07-27 23:09 对不起，刚回，说两句》
""",

    "50": """
市场就是这样的一个生物链，本ID这种第一波进去的，等着4000点上进去的抬，而4000点进去的，
顶住4300点，自然有新一波人进来。所以，现在4000点进来的也进退自如了，当然，如本ID者，
就更自如了，上面已经有了两层保护膜。

--摘自《2007-07-30 15:19 先扬后抑，酝酿震荡》
""",

    "51": """
市场充满了无穷的诱惑与陷阱，对应着人的贪婪与恐惧。单纯停留在技术的层面，最多就是一个交
易机器。

投资市场最终比的是修养与人格及见识，光从技艺上着手，永远只能是匠人，不可能成为真正的高
手。古代有所谓的打禅七，在现代社会，能找到7天来打禅七是极其奢侈的事情了。但每周，有一
个小时，抛开一切束缚，抛开一切人群，独自一个人，在房间里、在高山上、在河流里、在星空下
、在山野的空谷回音中，张开没有眼睛的眼睛、没有耳朵的耳朵、俯视这世界、倾听这世界。其实
，何处不是房间、高山、河流、星空、山野？何处有束缚需要抛开？在资本、政治、淫乱贪婪、恐
惧的血盆大口里，就是自由、解脱的清凉之地。当然，如果没有如此见识，还是先去需要自己的房
间、高山、河流、星空、山野，但最终，依然要在五浊恶世中污之恶之，不如此，无以自由、解脱。

--摘自《2007-01-15 15:50 教你炒股票23：市场与人生》
""",

    "52": """
市场没有什么庄家，有的只是赢家和输家！有的只是各种类型的动物，还有极少数的高明猎手。市场
就是一个围猎的游戏，当你只有一把小弓箭，你可以去打野兔；当你有了屠龙刀，抓几条蛇来玩当然
就没劲了，关键你是否有屠龙刀！

--摘自《2006-06-07 22:41 教你炒股票2：没有庄家，有的只是赢家和输家！》
""",

    "53": """
禅者，即迷非迷、即缠非缠，非悟即悟、非解即解。求解脱者无解脱、不求解脱者也无解脱，因解成
缠、因悟成迷。所谓顿悟者，非顿悟而名顿悟。所谓三关者，非三关而名三关。佛法无多子、禅宗无
多子、乾坤今古无多子，虽如此，此间事犹需一一透脱。所谓透脱者，非透脱而名透脱也。

--摘自《2006-09-28 11:50 缠中说禅：缠非缠、禅非禅，枯木龙吟照大千（序-九）修改版本》
""",

    "54": """
走势反映的是人的贪嗔痴疑慢，如果你能通过走势当下的呈现，而观照其中参与的心理呈现，就等于
看穿了市场参与者的内心。心理，不是虚无飘渺的，最终必然要留下痕迹，也就是市场走势本身。而
一些具有自相似性的结构，就正好是窥测市场心理的科学仪器。

--摘自《2007-09-24 21:31 教你炒股票82：分型结构的心理因素》
""",

    "55": """
本ID理论的哲学本质，就在于人的贪嗔痴疑慢所引发的自相似性以及由此引发走势级别的自组性这
种类生命的现象。走势是有生命的，本ID说“看行情的走势，就如同听一朵花的开放，见一朵花的
芬芳，嗅一朵花的美丽，一切都在当下中灿烂”，这绝对不是孔男人式的矫情比喻，而是科学般的严
谨说明，因为走势确实有着如花一般的生命特征，走势确实在自相似性、自组性中发芽、生长、绽
放、凋败。因此，本ID的理论是一种可发展的理论，可以提供给无数人去不断研究，研究的方向是
什么？就是走势的自相似性、自组性。

--摘自《2007-09-17 22:57 教你炒股票81：图例、更正及分型、走势类型的哲学本质》
""",

    "56": """
为什么要研究分型、走势类型等东西，其哲学基础是什么？这就是人的贪嗔痴疑慢。因为人的贪嗔痴
疑慢都是一样的，只是跟随时间、环境大小不一，所以，就显示出自相似性。而走势是所有人贪嗔痴
疑慢的合力结果，反映在走势中，就使得走势显示出自相似性。

--摘自《2007-09-17 22:57 教你炒股票81：图例、更正及分型、走势类型的哲学本质》
""",

    "57": """
分型、走势类型的本质就是自相似性，同样，走势必完美的本质也就是自相似性。分型，在1分钟级
别是这样的结构，在年线上也是这样的结构，在不同的级别上，级别不同，但结构是一样的，这就是
自相似性。同样，走势类型也一样。正因为走势具有自相似性，所以走势才是可理解的，才是可把握
的，如果没有自相似性，那么走势必然不可理解，无法把握。要把握走势，本质上，就是把握其自相
似性。

--摘自《2007-09-17 22:57 教你炒股票81：图例、更正及分型、走势类型的哲学本质》
""",


}

def print_one():
    k = random.choice(list(texts.keys()))
    print(texts[k].strip())



# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2021/8/3 17:10
describe: 掘金量化实盘utils
"""
import dill
import time
from pprint import pprint
from czsc.utils.cache import home_path

from ..monitor import stocks_monitor_rt
from ..selector import stocks_dwm_selector_rt
from .base import *


def on_order_status(context, order):
    """
    https://www.myquant.cn/docs/python/python_object_trade#007ae8f5c7ec5298

    :param context:
    :param order:
    :return:
    """
    latest_dt = context.now.strftime("%Y-%m-%d %H:%M:%S")
    logger = context.logger
    symbol = order.symbol
    if symbol not in context.symbols_map.keys():
        context.logger.warn(f"{symbol} on_order_status error")
        return

    trader = context.symbols_map[symbol]['trader']
    file_orders = context.file_orders
    msg = f"订单状态更新通知：\n{'*' * 31}\n" \
          f"时间：{latest_dt}\n" \
          f"标的：{order.symbol}\n" \
          f"名称：{context.shares.get(order.symbol, '')}\n" \
          f"操作：{trader.op['operate'] + '#' + trader.cache['last_op_desc']}\n" \
          f"方向：{order.side}\n" \
          f"价格：{round(order.price, 2)}\n" \
          f"状态：{order.status}\n" \
          f"委托量：{int(order.volume)}\n" \
          f"已成量：{int(order.filled_volume)}\n" \
          f"均价：{round(order.filled_vwap, 2)}"

    logger.info(msg.replace("\n", " - "))
    if context.mode == MODE_BACKTEST:
        with open(file_orders, 'a', encoding="utf-8") as f:
            f.write(str(order) + '\n')
    else:
        if order.status in [1, 3]:
            push_text(content=str(msg), key=context.wx_key)


def on_execution_report(context, execrpt):
    """响应委托被执行事件，委托成交或者撤单拒绝后被触发。

    https://www.myquant.cn/docs/python/python_trade_event#on_execution_report%20-%20%E5%A7%94%E6%89%98%E6%89%A7%E8%A1%8C%E5%9B%9E%E6%8A%A5%E4%BA%8B%E4%BB%B6
    https://www.myquant.cn/docs/python/python_object_trade#ExecRpt%20-%20%E5%9B%9E%E6%8A%A5%E5%AF%B9%E8%B1%A1

    :param context:
    :param execrpt:
    :return:
    """
    latest_dt = context.now.strftime(dt_fmt)
    logger = context.logger
    msg = f"委托订单被执行通知：\n{'*' * 31}\n" \
          f"时间：{latest_dt}\n" \
          f"标的：{execrpt.symbol}\n" \
          f"方向：{execrpt.side}\n" \
          f"成交量：{int(execrpt.volume)}\n" \
          f"成交价：{round(execrpt.price, 2)}\n" \
          f"执行回报类型：{execrpt.exec_type}"

    logger.info(msg.replace("\n", " - "))
    if context.mode != MODE_BACKTEST and execrpt.exec_type in [1, 5, 6, 8, 12, 19]:
        push_text(content=str(msg), key=context.wx_key)


def on_error(context, code, info):
    logger = context.logger
    msg = "{} - {}".format(code, info)
    logger.warn(msg)
    if context.mode != MODE_BACKTEST:
        push_text(content=msg, key=context.wx_key)


def on_account_status(context, account):
    """响应交易账户状态更新事件，交易账户状态变化时被触发"""
    print(str(account))


def adjust_future_position_rt(context, symbol: str, trader: CzscTrader, mp: float):
    """调整单只标的仓位

    :param mp: 开仓比例限制
    :param trader:
    :param context:
    :param symbol: 交易标的
    :return:
    """
    assert 1 >= mp >= 0, "开仓限制错误"

    if context.mode == MODE_BACKTEST:
        account = context.account()
    else:
        account = context.account(account_id=context.future_id)

    # 判断是否需要平多仓
    long_position = account.positions(symbol=symbol, side=PositionSide_Long)
    if long_position:
        lp = long_position[0].available
        oe = is_order_exist(context, symbol, OrderSide_Sell, PositionEffect_Close)
        if not oe and lp > 0 and trader.op['operate'] == Operate.LE.value:
            context.logger.info("{} - 平多 - {} - {}".format(symbol, trader.op['desc'], trader.latest_price))
            write_bs(context, symbol, trader.op)
            order_volume(symbol=symbol, volume=lp, side=OrderSide_Sell, order_type=OrderType_Market,
                         position_effect=PositionEffect_Close, account=account.id)
            take_snapshot(context, trader, name=trader.op['desc'])

    # 判断是否需要平空仓
    short_position = account.positions(symbol=symbol, side=PositionSide_Short)
    if short_position:
        sp = short_position[0].available
        oe = is_order_exist(context, symbol, OrderSide_Buy, PositionEffect_Close)
        if not oe and sp > 0 and trader.op['operate'] == Operate.SE.value:
            context.logger.info("{} - 平空 - {} - {}".format(symbol, trader.op['desc'], trader.latest_price))
            write_bs(context, symbol, trader.op)
            order_volume(symbol=symbol, volume=sp, side=OrderSide_Buy, order_type=OrderType_Market,
                         position_effect=PositionEffect_Close, account=account.id)
            take_snapshot(context, trader, name=trader.op['desc'])

    # 判断是否需要开多仓
    if not long_position and trader.op['operate'] == Operate.LO.value:
        oe = is_order_exist(context, symbol, OrderSide_Buy, PositionEffect_Open)
        if not oe:
            context.logger.info("{} - 开多 - {} - {}".format(symbol, trader.op['desc'], trader.latest_price))
            if mp >= 1:
                order_volume(symbol=symbol, volume=mp, side=OrderSide_Buy, order_type=OrderType_Market,
                             position_effect=PositionEffect_Open, account=account.id)
            else:
                order_target_percent(symbol=symbol, percent=mp, position_side=PositionSide_Long,
                                     order_type=OrderType_Market, account=account.id)
            take_snapshot(context, trader, name=trader.op['desc'])
            write_bs(context, symbol, trader.op)

    # 判断是否需要开空仓
    if not short_position and trader.op['operate'] == Operate.SO.value:
        oe = is_order_exist(context, symbol, OrderSide_Sell, PositionEffect_Open)
        if not oe:
            context.logger.info("{} - 开空 - {} - {}".format(symbol, trader.op['desc'], trader.latest_price))
            if mp >= 1:
                order_volume(symbol=symbol, volume=mp, side=OrderSide_Sell, order_type=OrderType_Market,
                             position_effect=PositionEffect_Open, account=account.id)
            else:
                order_target_percent(symbol=symbol, percent=mp, position_side=PositionSide_Short,
                                     order_type=OrderType_Market, account=account.id)
            take_snapshot(context, trader, name=trader.op['desc'])
            write_bs(context, symbol, trader.op)


def adjust_share_position_rt(context, symbol: str, trader: CzscTrader, mp: float, T0=False):
    """调整单只标的仓位

    :param T0: 是否允许 T+0 交易，默认为 False，表示不允许 T+0
    :param mp: 开仓比例限制
    :param trader:
    :param context:
    :param symbol: 交易标的
    :return:
    """
    assert 1 >= mp >= 0, "开仓限制错误"
    if symbol[:7] in ['SZSE.12', 'SHSE.11']:
        assert T0, "可转债支持 T+0 操作"

    in_trade_time = "14:59" > context.now.strftime("%H:%M") > "09:31"
    if not in_trade_time:
        return

    name = context.shares.get(symbol, "")
    if context.mode == MODE_BACKTEST:
        account = context.account()
    else:
        account = context.account(account_id=context.share_id)

    long_position = account.positions(symbol=symbol, side=PositionSide_Long)
    if long_position:
        # 判断是否需要平多仓
        if T0:
            lp = long_position[0].volume
        else:
            lp = long_position[0].volume - long_position[0].volume_today

        oe = is_order_exist(context, symbol, OrderSide_Sell, PositionEffect_Close)

        if not oe and trader.op['operate'] in [Operate.LE.value, Operate.HO.value]:
            trader.op['desc'] = trader.cache['last_op_desc']
            if lp <= 0:
                context.logger.info(
                    "{} - {} - 可平仓位为零 - {} - {}".format(symbol, name, trader.op['desc'], trader.latest_price))
            else:
                context.logger.info(
                    "{} - {} - 平多 - {} - {}".format(symbol, name, trader.op['desc'], trader.latest_price))
                order_volume(symbol=symbol, volume=lp, side=OrderSide_Sell, order_type=OrderType_Market,
                             position_effect=PositionEffect_Close, account=account.id)
                take_snapshot(context, trader, name=trader.op['desc'])
                write_bs(context, symbol, trader.op)
    else:
        # 判断是否需要开多仓
        oe = is_order_exist(context, symbol, OrderSide_Buy, PositionEffect_Open)

        if not oe and trader.op['operate'] in [Operate.LO.value, Operate.HL.value]:
            trader.op['desc'] = trader.cache['last_op_desc']
            context.logger.info("{} - {} - 开多 - {} - {}".format(symbol, name, trader.op['desc'], trader.latest_price))

            if trader.kg.m1[-1].id - trader.cache['long_open_k1_id'] < context.wait_time:
                context.logger.info(f"{symbol}，开仓条件满足，等待开仓中，"
                                    f"已等待 {trader.kg.m1[-1].id - trader.cache['long_open_k1_id']}")
                return

            # 判断标的是否在允许开仓的股票列表中
            if symbol not in context.allow_open_shares:
                context.logger.info(f"{symbol} - {name} 不在允许开仓的股票列表中")
                return

            # 判断当前价格是否在开仓容差范围
            if abs(trader.latest_price - trader.cache['long_open_price']) > trader.cache['long_open_price'] * 0.03:
                context.logger.info(f"{symbol}，当前价（{trader.latest_price}）不在"
                                    f"开仓价（{trader.cache['long_open_price']}）容差范围，不允许开仓")
                return

            # 判断总仓位是否已经达到限制水平
            if 1 - account.cash.available / account.cash.nav > context.max_total_position:
                context.logger.info(f"当前持仓比例已经超过{context.max_total_position * 100}%，不允许再开仓")
                return

            # 判断是否有足够的资金用来开仓
            if account.cash.available < max(105 * trader.latest_price, 20000):
                context.logger.info(
                    "{} - {} - 开多信号触发 - {} - {}；可用资金小于2万 / 可用仓位不够开1手，暂不开仓".format(
                        symbol, name, trader.op['desc'], trader.latest_price))
                return

            if mp >= 1:
                max_mp = account.cash.available // (100 * trader.latest_price)
                mp = min(mp, max_mp)
                order_volume(symbol=symbol, volume=mp * 100, side=OrderSide_Buy, order_type=OrderType_Limit,
                             price=trader.latest_price, position_effect=PositionEffect_Open, account=account.id)
            else:
                max_mp = (account.cash.available * 0.95) / account.cash.nav
                mp = min(max_mp, mp)
                # 实盘陷阱：市价柜台是按照涨停价冻结资金的
                # 修改成 按最近一个1分钟bar的close开仓
                order_target_percent(symbol=symbol, percent=mp, position_side=PositionSide_Long,
                                     order_type=OrderType_Limit, price=trader.latest_price, account=account.id)
            take_snapshot(context, trader, name=trader.op['desc'])
            write_bs(context, symbol, trader.op)


def adjust_position_rt(context, symbol):
    bars = context.data(symbol=symbol, frequency='60s', count=100, fields='symbol,eob,open,close,high,low,volume')
    trader: CzscTrader = context.symbols_map[symbol]['trader']
    mp = context.symbols_map[symbol]['mp']
    bars = format_kline(bars, freq=Freq.F1)
    bars_new = [x for x in bars if x.dt > trader.kg.end_dt]
    if bars_new:
        for k in bars_new:
            trader.check_operate(k, context.stoploss, context.timeout, context.wait_time, context.max_open_tolerance)
    context.symbols_map[symbol]['trader'] = trader
    context.logger.info(context.shares.get(symbol, "无名标的") + " : op    : " + str(trader.op))
    context.logger.info(context.shares.get(symbol, "无名标的") + " : cache : " + str(dict(trader.cache)))

    if symbol in context.forbidden:
        context.logger.info(context.shares.get(symbol, "无名标的") + " 禁止交易")
        return

    # 可转债支持 T+0 交易
    if symbol[:7] in ['SZSE.12', 'SHSE.11']:
        adjust_share_position_rt(context, symbol, trader, mp, T0=True)
        return

    exchange, sec_code = symbol.split(".")
    if exchange in ["SZSE", "SHSE"]:
        adjust_share_position_rt(context, symbol, trader, mp, T0=False)
    else:
        adjust_future_position_rt(context, symbol, trader, mp)

    # 每隔30分钟保存一次trader对象
    if bars[-1].dt.minute % 30 == 0:
        file_gt = os.path.join(context.data_path, "{}.gt".format(symbol))
        dill.dump(trader, open(file_gt, 'wb'))
        context.logger.info(f'{symbol} trader cached into {file_gt}')


def on_bar(context, bars):
    context.unfinished_orders = get_unfinished_orders()
    cancel_timeout_orders(context, max_m=10)

    for bar in bars:
        symbol = bar['symbol']
        try:
            adjust_position_rt(context, symbol)
        except:
            traceback.print_exc()


def report_stocks_account_status(context):
    """报告股票账户状态"""
    if context.now.isoweekday() > 5:
        print(f"{context.now} 不是交易时间")
        return

    if not context.share_id:
        push_text("股票账户ID为空，无法获取账户状态报告。", key=context.wx_key)
        return

    account = context.account(account_id=context.share_id)
    cash = account.cash
    positions = account.positions(symbol="", side="")

    msg = f"股票账户状态报告\n{'*' * 31}\n"
    msg += f"账户净值：{int(cash.nav)}\n" \
           f"持仓市值：{int(cash.market_value)}\n" \
           f"可用资金：{int(cash.available)}\n" \
           f"浮动盈亏：{int(cash.fpnl)}\n" \
           f"标的数量：{len(positions)}\n" \
           f"{'*' * 31}\n"

    for p in positions:
        try:
            msg += f'标的代码：{p.symbol}\n' \
                   f"标的名称：{context.shares.get(p.symbol, '')}\n" \
                   f'持仓数量：{p.volume}\n' \
                   f'最新价格：{round(p.price, 2)}\n' \
                   f'持仓成本：{round(p.vwap, 2)}\n' \
                   f'盈亏比例：{int((p.price - p.vwap) / p.vwap * 10000) / 100}%\n' \
                   f'持仓市值：{int(p.volume * p.vwap)}\n' \
                   f'持仓天数：{(context.now - p.created_at).days}\n' \
                   f"{'*' * 31}\n"
        except:
            print(p)

    push_text(msg.strip("\n *"), key=context.wx_key)


def is_order_exist(context, symbol, side, position_effect) -> bool:
    """判断同类型订单是否已经存在

    :param context:
    :param symbol: 交易标的
    :param side: 交易方向
    :param position_effect: 开平标志
    :return: bool
    """
    uo = context.unfinished_orders
    if not uo:
        return False
    else:
        for o in uo:
            if o.symbol == symbol and o.side == side and o.position_effect == position_effect:
                context.logger.info("同类型订单已存在：{} - {} - {}".format(symbol, side, position_effect))
                return True
    return False


def cancel_timeout_orders(context, max_m=10):
    """实盘仿真，撤销挂单时间超过 max_m 分钟的订单。

    :param context:
    :param max_m: 最大允许挂单分钟数
    :return:
    """
    for u_order in context.unfinished_orders:
        if context.ipo_shares and u_order.symbol in context.ipo_shares:
            # 跳过新股，新股申购订单不能撤
            continue

        if context.now - u_order.created_at >= timedelta(minutes=max_m):
            order_cancel(u_order)
            msg = "撤单通知：\n{}\n标的：{}\n".format("*" * 31, u_order.symbol)
            msg += "价格：{}\n时间：{}\n".format(u_order.price, u_order.created_at.strftime(dt_fmt))
            msg += "开平标志：{}\n买卖方向：{}".format(u_order.position_effect, u_order.side)
            context.logger.info(msg.replace("\n", " - "))
            if context.mode != MODE_BACKTEST:
                push_text(msg, context.wx_key)


def send_ipo_order(context):
    """实盘申购新股
    ipo_get_lot_info    https://www.myquant.cn/docs/python/python_select_api#0cbdf124a950907e
    ipo_get_instruments https://www.myquant.cn/docs/python/python_select_api#0cbdf124a950907e
    ipo_get_quota       https://www.myquant.cn/docs/python/python_select_api#0cbdf124a950907e
    ipo_buy             https://www.myquant.cn/docs/python/python_trade_api#31223e239affb510
    :param context:
    :return:
    """
    new_shares = ipo_get_instruments(sec_type=SEC_TYPE_STOCK, account_id=context.share_id, df=False)
    print(new_shares)
    if not new_shares:
        return
    context.ipo_shares = [x['symbol'] for x in new_shares]
    quota = ipo_get_quota(context.share_id)

    # ipo_buy(symbol, volume, price, account_id)


def push_operate_status(context):
    """推送全部交易标的操作状态"""
    if context.now.isoweekday() > 5:
        print(f"{context.now} 不是交易时间")
        return

    try:
        msg = "## 全部交易标的多空状态\n\n"
        i = 0
        for symbol, v in context.symbols_map.items():
            trader: CzscTrader = v['trader']
            last_op = trader.cache['last_op']
            last_op_desc = trader.cache['last_op_desc']

            if last_op in [Operate.HL.value, Operate.HS.value]:
                i += 1
                msg += f">{i}. {symbol} - {context.shares.get(symbol, '无名')} - {last_op} - {last_op_desc}\n"

        push_markdown(msg.strip("\n"), context.wx_key)
    except:
        print("push_operate_status fail.")
        traceback.print_exc()


def init_context_rt(context, name, symbols,
                    op_freq: Freq,
                    freqs: List[str],
                    get_signals: Callable,
                    get_events: Callable):
    """实盘 context 初始化方法

    :param context:
    :param name: tactic 的名称
    :param symbols: 交易列表
    :param op_freq: 交易周期
    :param freqs: 多级别周期
    :param get_signals: 信号计算方法
    :param get_events: 交易事件定义
    :return:
    """
    assert context.mode != MODE_BACKTEST
    path_gm_logs = os.environ.get('path_gm_logs', None)
    assert os.path.exists(path_gm_logs)
    data_path = os.path.join(path_gm_logs, f"realtime/{name}")
    cache_path = os.path.join(data_path, "cache")
    os.makedirs(cache_path, exist_ok=True)
    context.data_path = data_path
    context.cache_path = cache_path

    context.logger = create_logger(os.path.join(data_path, "realtime.log"), cmd=True, name="gm")
    context.file_orders = os.path.join(data_path, "orders.txt")
    context.share_id = os.environ['share_id']
    context.future_id = None
    context.shares = get_shares()

    context.op_freq = op_freq
    context.freqs = freqs
    context.get_signals = get_signals
    context.get_events = get_events

    context.wx_key = os.environ['wx_key']
    context.stoploss = float(os.environ['stoploss'])  # 止损条件设定
    context.timeout = int(os.environ['timeout'])      # 超时条件设定
    context.wait_time = int(os.environ['wait_time'])  # 开仓等待时长，单位：分钟
    context.max_open_tolerance = float(os.environ['max_open_tolerance'])    # 最大开仓容错百分比

    context.ipo_shares = []
    # 仓位控制[0, 1]，按资金百分比控制，1表示满仓，仅在开仓的时候控制
    context.max_total_position = float(os.environ['max_total_position'])
    context.max_share_position = float(os.environ['max_share_position'])
    assert 0 <= context.max_total_position <= 1
    assert 0 <= context.max_share_position <= 1
    assert context.max_total_position >= context.max_share_position >= 0

    context.logger.info("=" * 88)
    context.logger.info("实盘配置：")
    context.logger.info(f"总仓位控制：{context.max_total_position}")
    context.logger.info(f"单仓位控制：{context.max_share_position}")
    context.logger.info(f"异常退出条件：stoploss = {context.stoploss}; timeout = {context.timeout}")
    context.logger.info(f"K线交易周期：{context.op_freq}")
    context.logger.info(f"K线周期列表：{context.freqs}")
    context.logger.info("交易信号计算：\n" + inspect.getsource(context.get_signals))
    context.logger.info("交易事件定义：\n" + inspect.getsource(context.get_events))

    # 仅允许在最近设置的股票池列表中的股票上开仓
    context.allow_open_shares = list(symbols)

    # 查询当前持仓，保证持仓标的在交易标的列表中
    positions = context.account(account_id=context.share_id).positions(symbol="", side="")
    if positions:
        p_symbols = [p.symbol for p in positions]
        symbols += p_symbols

    symbols = sorted(list(set(symbols)))
    context.logger.info(f"交易标的数量：{len(symbols)}")
    symbols_map = {symbol: dict() for symbol in symbols}

    for symbol in symbols_map.keys():
        try:
            symbols_map[symbol]['mp'] = context.max_share_position
            file_gt = os.path.join(data_path, "{}.gt".format(symbol))
            if os.path.exists(file_gt) and time.time() - os.path.getmtime(file_gt) < 3600 * 72:
                trader: CzscTrader = dill.load(open(file_gt, 'rb'))
                context.logger.info("{} 加载成功，最新K线时间：{}".format(file_gt, trader.end_dt.strftime(dt_fmt)))
            else:
                kg = get_init_kg(symbol, context.now, max_count=2000, adjust=ADJUST_PREV, freqs=freqs)
                trader = CzscTrader(kg=kg, get_signals=get_signals, events=get_events(), op_freq=op_freq)

                # 更新多头持仓信息
                p = context.account(account_id=context.share_id).positions(symbol=symbol, side=PositionSide_Long)
                if p:
                    assert len(p) == 1
                    trader.cache['last_op'] = Operate.HL.value
                    trader.cache['last_op_desc'] = "启动时持多仓"
                    trader.cache['long_open_price'] = p[0].vwap
                    trader.cache['long_open_error_price'] = p[0].vwap * 0.97
                    trader.cache['long_open_k1_id'] = trader.kg.m1[-1].id
                    trader.cache['long_max_high'] = max(trader.latest_price, p[0].vwap)
                    context.logger.info(f'{symbol} 多头持仓信息更新成功：{dict(trader.cache)}')

                symbols_map[symbol]['trader'] = trader
                context.logger.info("{} 初始化完成，最新K线时间：{}".format(symbol, trader.end_dt.strftime(dt_fmt)))
                dill.dump(trader, open(file_gt, 'wb'))

            symbols_map[symbol]['trader'] = trader
        except:
            del symbols_map[symbol]
            print(f'init kg fail on {symbol}')

    subscribe(",".join(symbols_map.keys()), frequency='60s', count=300, wait_group=False)
    pprint(symbols_map)
    context.symbols_map = symbols_map

    # 设置定时任务
    # ------------------------------------------------------------------------------------------------------------------
    schedule(schedule_func=report_stocks_account_status, date_rule='1d', time_rule='09:31:00')
    schedule(schedule_func=report_stocks_account_status, date_rule='1d', time_rule='10:01:00')
    schedule(schedule_func=report_stocks_account_status, date_rule='1d', time_rule='10:31:00')
    schedule(schedule_func=report_stocks_account_status, date_rule='1d', time_rule='11:01:00')
    schedule(schedule_func=report_stocks_account_status, date_rule='1d', time_rule='11:31:00')
    schedule(schedule_func=report_stocks_account_status, date_rule='1d', time_rule='13:01:00')
    schedule(schedule_func=report_stocks_account_status, date_rule='1d', time_rule='13:31:00')
    schedule(schedule_func=report_stocks_account_status, date_rule='1d', time_rule='14:01:00')
    schedule(schedule_func=report_stocks_account_status, date_rule='1d', time_rule='14:31:00')
    schedule(schedule_func=report_stocks_account_status, date_rule='1d', time_rule='15:01:00')
    # schedule(schedule_func=stocks_dwm_selector_rt, date_rule='1d', time_rule='17:05:00')
    # schedule(schedule_func=stocks_monitor_rt, date_rule='1d', time_rule='09:05:00')
    # schedule(schedule_func=stocks_monitor_rt, date_rule='1d', time_rule='11:35:00')
    # schedule(schedule_func=stocks_monitor_rt, date_rule='1d', time_rule='15:05:00')


def check_index_status(qywx_key=None):
    """查看指数状态"""
    for gm_symbol in indices.values():
        try:
            file_html = os.path.join(home_path, f"{gm_symbol}.html")
            gm_take_snapshot(gm_symbol, end_dt=datetime.now(), file_html=file_html)
            if qywx_key:
                push_file(file_html, qywx_key)
        except:
            traceback.print_exc()




# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2021/8/3 17:25
"""
from tqdm import tqdm
from czsc.objects import Signal, Factor
from czsc.analyze import CZSC
from czsc import signals
from czsc.utils.io import read_pkl, save_pkl
from czsc.utils.kline_generator import KlineGeneratorD
from scripts_gm.src.utils.base import *


def stocks_dwm_selector(event: Event, get_signals: Callable,
                        max_count: int = 3000,
                        end_date: [str, datetime] = datetime.now(), wx_key=None):
    """大级别选股（日线&月线&周线）"""
    if isinstance(end_date, str):
        end_date = pd.to_datetime(end_date)
    kgd_path = "./data/kgd"
    os.makedirs(kgd_path, exist_ok=True)

    df = get_instruments(exchanges='SZSE,SHSE', sec_types=1, fields="symbol,sec_name", df=True)
    records = df.to_dict('records')

    home_path = os.path.expanduser('~')

    push_text(content="start running selector", key=wx_key)
    results = []
    for row in tqdm(records, desc=f"{end_date} - get signals"):
        symbol = row['symbol']
        try:
            file_kgd = os.path.join(kgd_path, f'{symbol}.kgd')
            if os.path.exists(file_kgd):
                kgd: KlineGeneratorD = read_pkl(file_kgd)
                if kgd.end_dt.date() != datetime.now().date() and datetime.now().isoweekday() <= 5:
                    k0 = get_kline(symbol, freq="1d", end_time=datetime.now(), count=1)
                    k0 = [x for x in k0 if x.dt > kgd.end_dt]
                    if k0:
                        print(k0)
                        for bar in k0:
                            kgd.update(bar)
            else:
                k0 = get_kline(symbol, end_time=end_date, freq='1d', count=10000, adjust=ADJUST_PREV)
                kgd = KlineGeneratorD()
                for bar in k0:
                    kgd.update(bar)
            save_pkl(kgd, file_kgd)
            # assert kgd.end_dt.date() == datetime.now().date(), f"kgd.end_dt = {kgd.end_dt}"

            last_vols = [k_.open * k_.vol for k_ in kgd.bars[Freq.D.value][-10:]]
            if min(last_vols) < 1e8:
                continue

            c0 = CZSC(kgd.bars[Freq.D.value][-max_count:], get_signals=get_signals)
            c1 = CZSC(kgd.bars[Freq.W.value][-max_count:], get_signals=get_signals)
            c2 = CZSC(kgd.bars[Freq.M.value][-max_count:], get_signals=get_signals)

            s = OrderedDict(row)
            s.update(c0.signals)
            s.update(c1.signals)
            s.update(c2.signals)

            m, f = event.is_match(s)
            if m:
                msg = f"{s['sec_name']}: {f}\n"
                msg += f"最新时间：{kgd.end_dt.strftime(dt_fmt)}\n"
                msg += f"同花顺F10：http://basic.10jqka.com.cn/{symbol.split('.')[1]}\n"
                msg += f"新浪行情：https://finance.sina.com.cn/realstock/company/{symbol[:2].lower()}{symbol[-6:]}/nc.shtml"
                push_text(content=msg, key=wx_key)

                res = {
                    'symbol': symbol,
                    'name': s['sec_name'],
                    'reason': f,
                    'end_dt': kgd.end_dt.strftime(dt_fmt),
                    'F10': f"http://basic.10jqka.com.cn/{symbol.split('.')[1]}",
                    'Kline': f"https://finance.sina.com.cn/realstock/company/{symbol[:2].lower()}{symbol[-6:]}/nc.shtml"
                }
                results.append(res)
                print(res)
        except:
            print("fail on {}".format(symbol))
            traceback.print_exc()

    file_results = os.path.join(home_path, f"selector_results_{end_date.strftime('%Y%m%d')}.xlsx")
    df_r = pd.DataFrame(results)
    df_r.to_excel(file_results, index=False)
    push_file(file_results, key=wx_key)
    push_text(content="end running selector", key=wx_key)


def stocks_dwm_selector_rt(context):
    if context.now.isoweekday() >= 6:
        return

    event = Event(name="选股", operate=Operate.LO, factors=[
        Factor(name="月线KDJ金叉_日线MACD强势", signals_all=[
            Signal("月线_KDJ状态_任意_金叉_任意_任意_0"),
            Signal('日线_MACD状态_任意_DIFF大于0_DEA大于0_柱子增大_0'),
            Signal('日线_MA5状态_任意_收盘价在MA5上方_任意_任意_0'),
        ]),
        Factor(name="月线KDJ金叉_日线潜在三买", signals_all=[
            Signal("月线_KDJ状态_任意_金叉_任意_任意_0"),
            Signal('日线_倒0笔_潜在三买_构成中枢_近3K在中枢上沿附近_近7K突破中枢GG_0'),
            Signal('日线_MA5状态_任意_收盘价在MA5上方_任意_任意_0'),
        ]),
        Factor(
            name="月线KDJ金叉_周线三笔强势",
            signals_all=[
                Signal("月线_KDJ状态_任意_金叉_任意_任意_0"),
                Signal('日线_MA5状态_任意_收盘价在MA5上方_任意_任意_0'),
            ],
            signals_any=[
                Signal('周线_倒1笔_三笔形态_向下不重合_任意_任意_0'),
                Signal('周线_倒1笔_三笔形态_向下奔走型_任意_任意_0'),
                Signal('周线_倒1笔_三笔形态_向下盘背_任意_任意_0'),
            ]
        ),
        Factor(name="月线KDJ金叉_周线MACD强势", signals_all=[
            Signal("月线_KDJ状态_任意_金叉_任意_任意_0"),
            Signal('周线_MACD状态_任意_DIFF大于0_DEA大于0_柱子增大_0'),
            Signal('日线_MA5状态_任意_收盘价在MA5上方_任意_任意_0'),
        ]),
    ])
    stocks_dwm_selector(event, signals.get_selector_signals, end_date=context.now, wx_key=context.wx_key)


if __name__ == '__main__':
    event = Event(name="选股", operate=Operate.LO, factors=[
        Factor(name="月线KDJ金叉_日线MACD强势", signals_all=[
            Signal("月线_KDJ状态_任意_金叉_任意_任意_0"),
            Signal('日线_MACD状态_任意_DIFF大于0_DEA大于0_柱子增大_0'),
            Signal('日线_MA5状态_任意_收盘价在MA5上方_任意_任意_0'),
        ]),
        Factor(name="月线KDJ金叉_日线潜在三买", signals_all=[
            Signal("月线_KDJ状态_任意_金叉_任意_任意_0"),
            Signal('日线_倒0笔_潜在三买_构成中枢_近3K在中枢上沿附近_近7K突破中枢GG_0'),
            Signal('日线_MA5状态_任意_收盘价在MA5上方_任意_任意_0'),
        ]),
        Factor(
            name="月线KDJ金叉_周线三笔强势",
            signals_all=[
                Signal("月线_KDJ状态_任意_金叉_任意_任意_0"),
                Signal('日线_MA5状态_任意_收盘价在MA5上方_任意_任意_0'),
            ],
            signals_any=[
                Signal('周线_倒1笔_三笔形态_向下不重合_任意_任意_0'),
                Signal('周线_倒1笔_三笔形态_向下奔走型_任意_任意_0'),
                Signal('周线_倒1笔_三笔形态_向下盘背_任意_任意_0'),
            ]
        ),
        Factor(name="月线KDJ金叉_周线MACD强势", signals_all=[
            Signal("月线_KDJ状态_任意_金叉_任意_任意_0"),
            Signal('周线_MACD状态_任意_DIFF大于0_DEA大于0_柱子增大_0'),
            Signal('日线_MA5状态_任意_收盘价在MA5上方_任意_任意_0'),
        ]),
    ])
    stocks_dwm_selector(event, signals.get_selector_signals, max_count=1000, wx_key='909731bd-****-46ad-****-24b9830873a4')



# coding: utf-8
import sys
import warnings
import os
import numpy as np
import pandas as pd
from copy import deepcopy

import czsc
from czsc.cobra.utils import down_cross_count, kdj_gold_cross, kdj_dead_cross
from czsc.cobra.utils import drop_duplicates_by_window
from czsc.objects import RawBar
from czsc.enum import Freq

warnings.warn(f"czsc version is {czsc.__version__}_{czsc.__date__}")

cur_path = os.path.split(os.path.realpath(__file__))[0]


def test_kdj_cross():
    file_kline = os.path.join(cur_path, "data/000001.SH_D.csv")
    kline = pd.read_csv(file_kline, encoding="utf-8")
    # bars = kline.to_dict("records")
    bars = [RawBar(symbol=row['symbol'], id=i, freq=Freq.D, open=row['open'], dt=row['dt'],
                   close=row['close'], high=row['high'], low=row['low'], vol=row['vol'])
            for i, row in kline.iterrows()]

    assert not kdj_gold_cross(kline, just=False)
    assert not kdj_gold_cross(bars, just=False)
    assert kdj_dead_cross(kline, just=False)
    assert kdj_dead_cross(bars, just=False)
    assert not kdj_dead_cross(kline, just=True)


def test_cross_count():
    x1 = [1, 1, 3, 4, 5, 12, 9, 8]
    x2 = [2, 2, 1, 5, 8, 9, 10, 10]
    assert down_cross_count(x1, x2) == 2
    assert down_cross_count(np.array(x1), np.array(x2)) == 2
    assert down_cross_count(x2, x1) == 2
    assert down_cross_count(np.array(x2), np.array(x1)) == 2


def test_drop_duplicates():
    seq1 = [1, 2, 3, 3, 2, 4, 5, 6, 8, 1]
    seq1_ = drop_duplicates_by_window(seq1, -1, window_size=5)
    assert seq1_[3] == -1
    assert seq1_[4] == -1

    seq2 = ['x1', 'x1', 'x2', 'x3', 'x4', 'x3', 'x2', 'x5', 'x6']
    seq2_ = drop_duplicates_by_window(deepcopy(seq2), 'other', window_size=5)
    assert seq2_[1] == seq2_[5] == seq2_[6] == 'other'
    seq2_ = drop_duplicates_by_window(deepcopy(seq2), 'other', window_size=3)
    assert seq2_[1] == seq2_[5] == 'other'

    seq3 = [0.1, 1.2, 2.3, 3.2, 1.2, 2.5, 0.1, 2.3]
    seq3_ = drop_duplicates_by_window(deepcopy(seq3), -0.1, window_size=5)
    assert seq3_[4] == -0.1
    seq3_ = drop_duplicates_by_window(deepcopy(seq3), -0.1, window_size=10)
    assert seq3_[4] == seq3_[6] == seq3_[7] == -0.1

    # 长序列性能：500 ms ± 3.14 ms per loop (mean ± std. dev. of 10 runs, 1 loop each)
    # seq = list(range(2000000))
    # %timeit -r 10 seq_ = drop_duplicates_by_window(seq, -1, window_size=5)



# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2021/09/23 11:51
"""


def create_logger(log_file, name='logger', cmd=True, level="info"):
    """define a logger for your program

    :param log_file: file name of log
    :param name: name of logger
    :param cmd: output in cmd
    :param level: level
    :return: logger
    """
    import logging

    level_map = {
        "info": logging.INFO,
        "debug": logging.DEBUG,
        "error": logging.ERROR,
    }
    log_level = level_map.get(level, logging.INFO)
    logger = logging.getLogger(name)
    logger.setLevel(log_level)

    # set format
    formatter = logging.Formatter('%(asctime)s | %(name)s | %(levelname)s | %(message)s',
                                  datefmt='%Y-%m-%d %H:%M:%S')

    # file handler
    fh = logging.FileHandler(log_file, encoding="utf-8")
    fh.setLevel(log_level)
    fh.setFormatter(formatter)
    logger.addHandler(fh)

    # cmd handler
    if cmd:
        ch = logging.StreamHandler()
        ch.setLevel(log_level)
        ch.setFormatter(formatter)
        logger.addHandler(ch)

    return logger






# coding: utf-8
import os
import pandas as pd
from tqdm import tqdm

from czsc.objects import RawBar, Freq
from czsc.utils.kline_generator import KlineGenerator, KlineGeneratorD
from test.test_analyze import read_1min

cur_path = os.path.split(os.path.realpath(__file__))[0]
kline = read_1min()


def test_kline_generator():
    # 验证指定级别
    bars = kline[:2000]

    kg = KlineGenerator(freqs=['周线', '日线', '30分钟', '5分钟', '1分钟'])
    for row in tqdm(bars):
        kg.update(row)

    assert not kg.m60 and not kg.m15 and not kg.M

    bars = kline[:20000]
    kg = KlineGenerator(max_count=2000)
    for row in tqdm(bars):
        kg.update(row)

    assert kg.m60 and kg.m15

    # 验证K线获取
    klines = kg.get_klines({'1分钟': 100, '5分钟': 100})
    for k, v in klines.items():
        assert len(v) == 100

    # 数量验证
    assert len(kg.m1) == 2000
    assert len(kg.m5) == 4001
    assert len(kg.m15) == 1334
    assert len(kg.m60) == 334
    assert len(kg.D) == 84
    assert len(kg.W) == 17
    assert len(kg.M) == 5

    # 测试实盘连续输入
    for _ in range(5):
        kg.update(bars[-1])
        assert len(kg.m1) == 2000


def test_kgd():
    df = pd.read_csv(os.path.join(cur_path, './data/000001.SH_D.csv'))
    bars = []
    for i, row in df.iterrows():
        bars.append(RawBar(symbol=row.symbol, id=i, freq=Freq.D, dt=pd.to_datetime(row['dt']),
                           open=row.open, close=row.close, high=row.high, low=row.low, vol=row.vol))
    kgd = KlineGeneratorD()
    for bar in bars:
        kgd.update(bar)

    assert kgd.end_dt == pd.to_datetime('2020-07-16 15:00:00')
    assert len(kgd.bars['月线']) == 165
    assert len(kgd.bars['年线']) == 15

    kgd = KlineGeneratorD(freqs=[Freq.D.value, Freq.W.value, Freq.M.value])
    for bar in bars:
        kgd.update(bar)

    assert kgd.end_dt == pd.to_datetime('2020-07-16 15:00:00')
    assert len(kgd.bars['月线']) == 165
    assert Freq.Y.value not in kgd.bars.keys()



# coding: utf-8
from dataclasses import dataclass
from datetime import datetime
from typing import List
from transitions import Machine

from .enum import Mark, Direction, Freq, Operate


@dataclass
class Tick:
    symbol: str
    name: str = ""
    price: float = 0
    vol: float = 0


@dataclass
class RawBar:
    """原始K线元素"""
    symbol: str
    id: int  # id 必须是升序
    dt: datetime
    freq: Freq
    open: [float, int]
    close: [float, int]
    high: [float, int]
    low: [float, int]
    vol: [float, int]
    amount: [float, int] = None


@dataclass
class NewBar:
    """去除包含关系后的K线元素"""
    symbol: str
    id: int  # id 必须是升序
    dt: datetime
    freq: Freq
    open: [float, int]
    close: [float, int]
    high: [float, int]
    low: [float, int]
    vol: [float, int]
    amount: [float, int] = None
    elements: List = None  # 存入具有包含关系的原始K线


@dataclass
class FX:
    symbol: str
    dt: datetime
    mark: Mark
    high: [float, int]
    low: [float, int]
    fx: [float, int]
    power: str = None
    elements: List = None


@dataclass
class FakeBI:
    """虚拟笔：主要为笔的内部分析提供便利"""
    symbol: str
    sdt: datetime
    edt: datetime
    direction: Direction
    high: [float, int]
    low: [float, int]
    power: [float, int]


@dataclass
class BI:
    symbol: str
    fx_a: FX = None  # 笔开始的分型
    fx_b: FX = None  # 笔结束的分型
    fxs: List = None  # 笔内部的分型列表
    direction: Direction = None
    high: float = None
    low: float = None
    power: float = None
    bars: List = None
    rsq: float = None
    change: float = None
    length: float = None
    fake_bis: List = None

    def __post_init__(self):
        self.sdt = self.fx_a.dt
        self.edt = self.fx_b.dt


@dataclass
class Signal:
    signal: str = None

    # score 取值在 0~100 之间，得分越高，信号越强
    score: int = 0

    # k1, k2, k3 是信号名称
    k1: str = "任意"
    k2: str = "任意"
    k3: str = "任意"

    # v1, v2, v3 是信号取值
    v1: str = "任意"
    v2: str = "任意"
    v3: str = "任意"

    # 任意 出现在模板信号中可以指代任何值

    def __post_init__(self):
        if not self.signal:
            self.signal = f"{self.k1}_{self.k2}_{self.k3}_{self.v1}_{self.v2}_{self.v3}_{self.score}"
        else:
            self.k1, self.k2, self.k3, self.v1, self.v2, self.v3, score = self.signal.split("_")
            self.score = int(score)

        if self.score > 100 or self.score < 0:
            raise ValueError("score 必须在0~100之间")

    def __repr__(self):
        return f"Signal('{self.signal}')"

    @property
    def key(self) -> str:
        """获取信号名称"""
        key = ""
        for k in [self.k1, self.k2, self.k3]:
            if k != "任意":
                key += k + "_"
        return key.strip("_")

    @property
    def value(self) -> str:
        """获取信号值"""
        return f"{self.v1}_{self.v2}_{self.v3}_{self.score}"

    def is_match(self, s: dict) -> bool:
        """判断信号是否与信号列表中的值匹配

        :param s: 所有信号字典
        :return: bool
        """
        key = self.key
        v = s.get(key, None)
        if not v:
            raise ValueError(f"{key} 不在信号列表中")

        v1, v2, v3, score = v.split("_")
        if int(score) >= self.score:
            if v1 == self.v1 or self.v1 == '任意':
                if v2 == self.v2 or self.v2 == '任意':
                    if v3 == self.v3 or self.v3 == '任意':
                        return True
        return False


@dataclass
class Factor:
    name: str
    # signals_all 必须全部满足的信号
    signals_all: List[Signal]
    # signals_any 满足其中任一信号，允许为空
    signals_any: List[Signal] = None

    def is_match(self, s: dict) -> bool:
        """判断 factor 是否满足"""
        for signal in self.signals_all:
            if not signal.is_match(s):
                return False

        if not self.signals_any:
            return True

        for signal in self.signals_any:
            if signal.is_match(s):
                return True
        return False


@dataclass
class Event:
    name: str
    operate: Operate

    # 多个信号组成一个因子，多个因子组成一个事件。
    # 单个事件是一系列同类型因子的集合，事件中的任一因子满足，则事件为真。
    factors: List[Factor]

    def is_match(self, s: dict):
        """判断 event 是否满足"""
        for factor in self.factors:
            if factor.is_match(s):
                # 顺序遍历，找到第一个满足的因子就退出。建议因子列表按关注度从高到低排序
                return True, factor.name

        return False, None

class PositionLong:
    def __init__(self, symbol: str,
                 hold_long_a: float = 0.5,
                 hold_long_b: float = 0.8,
                 hold_long_c: float = 1.0,
                 ):
        """多头持仓对象

        :param symbol: 标的代码
        :param hold_long_a: 首次开多仓后的仓位
        :param hold_long_b: 第一次加多后的仓位
        :param hold_long_c: 第二次加多后的仓位
        """
        assert 0 <= hold_long_a <= hold_long_b <= hold_long_c <= 1.0

        self.pos_changed = False
        self.symbol = symbol
        self.pos_map = {
            "hold_long_a": hold_long_a, "hold_long_b": hold_long_b,
            "hold_long_c": hold_long_c, "hold_money": 0
        }
        self.states = list(self.pos_map.keys())
        self.machine = Machine(model=self, states=self.states, initial='hold_money')
        self.machine.add_transition('long_open', 'hold_money', 'hold_long_a')
        self.machine.add_transition('long_add1', 'hold_long_a', 'hold_long_b')
        self.machine.add_transition('long_add2', 'hold_long_b', 'hold_long_c')
        self.machine.add_transition('long_reduce1', 'hold_long_c', 'hold_long_b')
        self.machine.add_transition('long_reduce2', ['hold_long_b', 'hold_long_c'], 'hold_long_a')
        self.machine.add_transition('long_exit', ['hold_long_a', 'hold_long_b', 'hold_long_c'], 'hold_money')

        self.operates = []
        self.long_high = -1         # 持多仓期间出现的最高价
        self.long_cost = -1         # 最近一次加多仓的成本
        self.long_bid = -1          # 最近一次加多仓的1分钟Bar ID

    @property
    def pos(self):
        """返回状态对应的仓位"""
        return self.pos_map[self.state]

    def evaluate_operates(self):
        """评估操作表现"""
        pass

    def update(self, dt: datetime, op: Operate, price: float, bid: int):
        """更新多头持仓状态

        :param dt: 最新时间
        :param op: 操作动作
        :param price: 最新价格
        :param bid: 最新1分钟Bar ID
        :return: None
        """
        state = self.state
        pos_changed = False

        if state == 'hold_money' and op == Operate.LO:
            self.long_open()
            pos_changed = True

        if state == 'hold_long_a' and op == Operate.LA1:
            self.long_add1()
            pos_changed = True

        if state == 'hold_long_b' and op == Operate.LA2:
            self.long_add2()
            pos_changed = True

        if state == 'hold_long_c' and op == Operate.LR1:
            self.long_reduce1()
            pos_changed = True

        if state in ['hold_long_b', 'hold_long_c'] and op == Operate.LR2:
            self.long_reduce2()
            pos_changed = True

        if state in ['hold_long_a', 'hold_long_b', 'hold_long_c'] and op == Operate.LE:
            self.long_exit()
            pos_changed = True

        if pos_changed:
            self.operates.append({
                "symbol": self.symbol,
                "dt": dt,
                "price": price,
                "bid": bid,
                "op": op
            })
        self.pos_changed = pos_changed

class Position:
    def __init__(self, symbol: str,
                 hold_long_a: float = 0.5,
                 hold_long_b: float = 0.8,
                 hold_long_c: float = 1.0,
                 hold_short_a: float = -0.5,
                 hold_short_b: float = -0.8,
                 hold_short_c: float = -1.0,
                 ):
        """持仓对象

        :param symbol: 标的代码
        :param hold_long_a: 首次开多仓后的仓位
        :param hold_long_b: 第一次加多后的仓位
        :param hold_long_c: 第二次加多后的仓位
        :param hold_short_a: 首次开空仓后的仓位
        :param hold_short_b: 第一次加空后的仓位
        :param hold_short_c: 第二次加空后的仓位
        """
        assert 0 <= hold_long_a <= hold_long_b <= hold_long_c <= 1.0
        assert 0 >= hold_short_a >= hold_short_b >= hold_short_c >= -1.0

        self.symbol = symbol
        self.pos_map = {
            "hold_long_a": hold_long_a, "hold_long_b": hold_long_b, "hold_long_c": hold_long_c, "hold_money": 0,
            "hold_short_a": hold_short_a,  "hold_short_b": hold_short_b,  "hold_short_c": hold_short_c
        }
        self.states = list(self.pos_map.keys())
        self.machine = Machine(model=self, states=self.states, initial='hold_money')
        self.machine.add_transition('long_open', 'hold_money', 'hold_long_a')
        self.machine.add_transition('long_add1', 'hold_long_a', 'hold_long_b')
        self.machine.add_transition('long_add2', 'hold_long_b', 'hold_long_c')
        self.machine.add_transition('long_reduce1', 'hold_long_c', 'hold_long_b')
        self.machine.add_transition('long_reduce2', ['hold_long_b', 'hold_long_c'], 'hold_long_a')
        self.machine.add_transition('long_exit', ['hold_long_a', 'hold_long_b', 'hold_long_c'], 'hold_money')

        self.machine.add_transition('short_open', 'hold_money', 'hold_short_a')
        self.machine.add_transition('short_add1', 'hold_short_a', 'hold_short_b')
        self.machine.add_transition('short_add2', 'hold_short_b', 'hold_short_c')
        self.machine.add_transition('short_reduce1', 'hold_short_c', 'hold_short_b')
        self.machine.add_transition('short_reduce2', ['hold_short_b', 'hold_short_c'], 'hold_short_a')
        self.machine.add_transition('short_exit', ['hold_short_a', 'hold_short_b', 'hold_short_c'], 'hold_money')

        self.operates = []
        self.long_high = -1         # 持多仓期间出现的最高价
        self.long_cost = -1         # 最近一次加多仓的成本
        self.long_bid = -1          # 最近一次加多仓的1分钟Bar ID

        self.short_low = -1             # 持空仓期间出现的最低价
        self.short_cost = -1            # 最近一次加空仓的成本
        self.short_bid = -1             # 最近一次加空仓的1分钟Bar ID

    @property
    def pos(self):
        """返回状态对应的仓位"""
        return self.pos_map[self.state]

    def update_long(self, dt: datetime, op: Operate, price: float, bid: int) -> bool:
        """更新多头持仓状态

        :param dt: 最新时间
        :param op: 操作动作
        :param price: 最新价格
        :param bid: 最新1分钟Bar ID
        :return: bool
        """
        state = self.state
        pos_changed = False

        if state == 'hold_money' and op == Operate.LO:
            self.long_open()
            return True

        if state == 'hold_long_a' and op == Operate.LA1:
            self.long_add1()
            return True

        if state == 'hold_long_b' and op == Operate.LA2:
            self.long_add2()
            return True

        if state == 'hold_long_c' and op == Operate.LR1:
            self.long_reduce1()
            return True

        if state in ['hold_long_b', 'hold_long_c'] and op == Operate.LR2:
            self.long_reduce2()
            return True

        if state in ['hold_long_a', 'hold_long_b', 'hold_long_c'] and op == Operate.LE:
            self.long_exit()
            return True

        return pos_changed

    def update_short(self, dt: datetime, op: Operate, price: float, bid: int) -> bool:
        """更新空头持仓状态

        :param dt: 最新时间
        :param op: 操作动作
        :param price: 最新价格
        :param bid: 最新1分钟Bar ID
        :return: bool
        """
        state = self.state
        pos_changed = False

        if state == 'hold_money' and op == Operate.SO:
            self.short_open()
            return True

        if state == 'hold_short_a' and op == Operate.SA1:
            self.short_add1()
            return True

        if state == 'hold_short_b' and op == Operate.SA2:
            self.short_add2()
            return True

        if state == 'hold_short_c' and op == Operate.SR1:
            self.short_reduce1()
            return True

        if state in ['hold_short_b', 'hold_short_c'] and op == Operate.SR2:
            self.short_reduce2()
            return True

        if state in ['hold_short_a', 'hold_short_b', 'hold_short_c'] and op == Operate.SE:
            self.short_exit()
            return True

        return pos_changed

    def update(self, dt: datetime, op: Operate, price: float, bid: int) -> bool:
        """更新多头持仓状态

        :param dt: 最新时间
        :param op: 操作动作
        :param price: 最新价格
        :param bid: 最新1分钟Bar ID
        :return: bool
        """
        if self.update_long(dt, op, price, bid):
            return True
        else:
            return self.update_short(dt, op, price, bid)





# coding: utf-8
from collections import OrderedDict
import pandas as pd
from czsc.objects import Signal, Factor, Event, Freq, Operate, Position, PositionLong


def test_signal():
    s = Signal(k1="1分钟", k3="倒1形态", v1="类一买", v2="七笔", v3="基础型", score=3)
    assert str(s) == "Signal('1分钟_任意_倒1形态_类一买_七笔_基础型_3')"
    assert s.key == "1分钟_倒1形态"
    s1 = Signal(signal='1分钟_任意_倒1形态_类一买_七笔_基础型_3')
    assert s == s1
    assert s.is_match({"1分钟_倒1形态": "类一买_七笔_基础型_3"})
    assert not s.is_match({"1分钟_倒1形态": "类一买_七笔_特例一_3"})
    assert not s.is_match({"1分钟_倒1形态": "类一买_九笔_基础型_3"})

    s = Signal(k1="1分钟", k2="倒1形态", k3="类一买", score=3)
    assert str(s) == "Signal('1分钟_倒1形态_类一买_任意_任意_任意_3')"
    assert s.key == "1分钟_倒1形态_类一买"

    try:
        s = Signal(k1="1分钟", k2="倒1形态", k3="类一买", score=101)
    except ValueError as e:
        assert str(e) == 'score 必须在0~100之间'


def test_factor():
    freq = Freq.F15
    s = OrderedDict()
    default_signals = [
        Signal(k1=str(freq.value), k2="倒0笔", k3="方向", v1="向上", v2='其他', v3='其他'),
        Signal(k1=str(freq.value), k2="倒0笔", k3="长度", v1="大于5", v2='其他', v3='其他'),
        Signal(k1=str(freq.value), k2="倒0笔", k3="三K形态", v1="顶分型", v2='其他', v3='其他'),

        Signal(k1=str(freq.value), k2="倒1笔", k3="表里关系", v1="其他", v2='其他', v3='其他'),
        Signal(k1=str(freq.value), k2="倒1笔", k3="RSQ状态", v1="小于0.2", v2='其他', v3='其他'),
    ]
    for signal in default_signals:
        s[signal.key] = signal.value

    factor = Factor(
        name="单测",
        signals_all=[
            Signal(k1=str(freq.value), k2="倒0笔", k3="方向", v1="向上", v2='其他', v3='其他'),
            Signal(k1=str(freq.value), k2="倒0笔", k3="长度", v1="大于5", v2='其他', v3='其他')
        ]
    )
    assert factor.is_match(s)

    factor = Factor(
        name="单测",
        signals_all=[
            Signal(k1=str(freq.value), k2="倒0笔", k3="方向", v1="向上", v2='其他', v3='其他'),
            Signal(k1=str(freq.value), k2="倒0笔", k3="长度", v1="大于5", v2='其他', v3='其他')
        ],
        signals_any=[
            Signal(k1=str(freq.value), k2="倒1笔", k3="RSQ状态", v1="小于0.2", v2='其他', v3='其他')
        ]
    )
    assert factor.is_match(s)

    factor = Factor(
        name="单测",
        signals_all=[
            Signal(k1=str(freq.value), k2="倒0笔", k3="方向", v1="向上", v2='其他', v3='其他'),
            Signal(k1=str(freq.value), k2="倒0笔", k3="长度", v1="大于5", v2='其他', v3='其他')
        ],
        signals_any=[
            Signal(k1=str(freq.value), k2="倒1笔", k3="RSQ状态", v1="小于0.8", v2='其他', v3='其他')
        ]
    )
    assert not factor.is_match(s)


def test_event():
    freq = Freq.F15
    s = OrderedDict()
    default_signals = [
        Signal(k1=str(freq.value), k2="倒0笔", k3="方向", v1="向上", v2='其他', v3='其他'),
        Signal(k1=str(freq.value), k2="倒0笔", k3="长度", v1="大于5", v2='其他', v3='其他'),
        Signal(k1=str(freq.value), k2="倒0笔", k3="三K形态", v1="顶分型", v2='其他', v3='其他'),

        Signal(k1=str(freq.value), k2="倒1笔", k3="表里关系", v1="其他", v2='其他', v3='其他'),
        Signal(k1=str(freq.value), k2="倒1笔", k3="RSQ状态", v1="小于0.2", v2='其他', v3='其他'),
    ]
    for signal in default_signals:
        s[signal.key] = signal.value

    event = Event(name="单测", operate=Operate.LO, factors=[
        Factor(
            name="测试",
            signals_all=[
                Signal(k1=str(freq.value), k2="倒0笔", k3="方向", v1="向上", v2='其他', v3='其他'),
                Signal(k1=str(freq.value), k2="倒0笔", k3="长度", v1="大于5", v2='其他', v3='其他')]
        )
    ])
    m, f = event.is_match(s)
    assert m and f

    event = Event(name="单测", operate=Operate.LO, factors=[
        Factor(
            name="测试",
            signals_all=[
                Signal('15分钟_倒0笔_方向_向上_其他_其他_0'), Signal('15分钟_倒0笔_长度_任意_其他_其他_0')
            ]
        )
    ])
    m, f = event.is_match(s)
    assert m and f

    event = Event(name="单测", operate=Operate.LO, factors=[
        Factor(
            name="测试",
            signals_all=[
                Signal('15分钟_倒0笔_方向_向上_其他_其他_20'), Signal('15分钟_倒0笔_长度_任意_其他_其他_0')
            ]
        )
    ])
    m, f = event.is_match(s)
    assert not m and not f

    event = Event(name="单测", operate=Operate.LO, factors=[
        Factor(
            name="测试",
            signals_all=[
                Signal('15分钟_倒0笔_方向_向下_其他_其他_0'), Signal('15分钟_倒0笔_长度_任意_其他_其他_0')
            ]
        )
    ])
    m, f = event.is_match(s)
    assert not m and not f


def test_position():
    position = Position(symbol="000001.XSHG")

    # 测试正常的完整多头流程
    assert position.pos == 0
    position.long_open()
    assert position.pos == 0.5
    position.long_add1()
    assert position.pos == 0.8
    position.long_add2()
    assert position.pos == 1
    position.long_reduce1()
    assert position.pos == 0.8
    position.long_reduce2()
    assert position.pos == 0.5
    position.long_exit()
    assert position.pos == 0

    # 测试正常的多头流程（直接 reduce2）
    assert position.pos == 0
    position.long_open()
    assert position.pos == 0.5
    position.long_add1()
    assert position.pos == 0.8
    position.long_add2()
    assert position.pos == 1
    position.long_reduce2()
    assert position.pos == 0.5
    position.long_exit()
    assert position.pos == 0

    # 测试多头只有一次开仓流程
    assert position.pos == 0
    position.long_open()
    assert position.pos == 0.5
    position.long_add1()
    assert position.pos == 0.8
    position.long_exit()
    assert position.pos == 0

    # 测试正常的完整空头流程
    assert position.pos == 0
    position.short_open()
    assert position.pos == -0.5
    position.short_add1()
    assert position.pos == -0.8
    position.short_add2()
    assert position.pos == -1
    position.short_reduce1()
    assert position.pos == -0.8
    position.short_reduce2()
    assert position.pos == -0.5
    position.short_exit()
    assert position.pos == 0

    # 测试正常的完整空头流程（直接 reduce2）
    assert position.pos == 0
    position.short_open()
    assert position.pos == -0.5
    position.short_add1()
    assert position.pos == -0.8
    position.short_add2()
    assert position.pos == -1
    position.short_reduce2()
    assert position.pos == -0.5
    position.short_exit()
    assert position.pos == 0


def test_position_long():
    pos_long = PositionLong(symbol="000001.XSHG")
    pos_long.update(dt=pd.to_datetime('2021-01-01'), op=Operate.HO, price=100, bid=0)
    assert not pos_long.pos_changed and pos_long.pos == 0

    pos_long.update(dt=pd.to_datetime('2021-01-02'), op=Operate.LO, price=100, bid=1)
    assert pos_long.pos_changed and pos_long.pos == 0.5

    pos_long.update(dt=pd.to_datetime('2021-01-03'), op=Operate.LO, price=100, bid=2)
    assert not pos_long.pos_changed and pos_long.pos == 0.5

    pos_long.update(dt=pd.to_datetime('2021-01-04'), op=Operate.LA1, price=100, bid=3)
    assert pos_long.pos_changed and pos_long.pos == 0.8

    pos_long.update(dt=pd.to_datetime('2021-01-05'), op=Operate.LA1, price=100, bid=4)
    assert not pos_long.pos_changed and pos_long.pos == 0.8

    pos_long.update(dt=pd.to_datetime('2021-01-06'), op=Operate.LA2, price=100, bid=5)
    assert pos_long.pos_changed and pos_long.pos == 1

    pos_long.update(dt=pd.to_datetime('2021-01-07'), op=Operate.LR1, price=100, bid=6)
    assert pos_long.pos_changed and pos_long.pos == 0.8

    pos_long.update(dt=pd.to_datetime('2021-01-08'), op=Operate.LR2, price=100, bid=7)
    assert pos_long.pos_changed and pos_long.pos == 0.5

    pos_long.update(dt=pd.to_datetime('2021-01-08'), op=Operate.LR2, price=100, bid=7)
    assert not pos_long.pos_changed and pos_long.pos == 0.5

    pos_long.update(dt=pd.to_datetime('2021-01-09'), op=Operate.LA2, price=100, bid=8)
    assert not pos_long.pos_changed and pos_long.pos == 0.5

    pos_long.update(dt=pd.to_datetime('2021-01-10'), op=Operate.LA1, price=100, bid=9)
    assert pos_long.pos_changed and pos_long.pos == 0.8

    pos_long.update(dt=pd.to_datetime('2021-01-11'), op=Operate.LE, price=100, bid=10)
    assert pos_long.pos_changed and pos_long.pos == 0




# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2021/8/3 17:09
"""
import czsc

version = "0.1.2"
update_date = '20210828'

# czsc 版本校验
assert czsc.__version__ >= '0.7.5'
assert czsc.__date__ >= '20210828'



# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2021/8/3 17:09
"""
import pandas as pd


def ths_symbol_to_gm(ths_symbol: str) -> str:
    """同花顺代码转掘金代码

    :param ths_symbol: 聚宽代码
    :return: gm_symbol
    """
    code, exchange = ths_symbol.split(".")
    if exchange == 'SH':
        gm_symbol = "SHSE." + code
    elif exchange == 'SZ':
        gm_symbol = "SZSE." + code
    else:
        raise ValueError
    return gm_symbol


def read_ths_zx(file_ths_zx: str) -> pd.DataFrame:
    """

    :param file_ths_zx: i问财导出的同花顺标的文件
    :return:
    """
    df = pd.read_excel(file_ths_zx)
    df = df.head(len(df)-1)
    df['掘金代码'] = df['股票代码'].apply(ths_symbol_to_gm)
    return df


def jq_symbol_to_gm(jq_symbol: str) -> str:
    """聚宽代码转掘金代码

    :param jq_symbol: 聚宽代码
    :return: gm_symbol
    """
    code, exchange = jq_symbol.split(".")
    if exchange == 'XSHG':
        gm_symbol = "SHSE." + code
    elif exchange == 'XSHE':
        gm_symbol = "SZSE." + code
    else:
        raise ValueError
    return gm_symbol


def gm_symbol_to_jq(gm_symbol: str) -> str:
    """掘金代码转聚宽代码

    :param gm_symbol: 掘金代码
    :return: jq_symbol
    """
    exchange, code = gm_symbol.split(".")
    if exchange == 'SHSE':
        jq_symbol = code + ".XSHG"
    elif exchange == 'SZSE':
        jq_symbol = code + ".XSHE"
    else:
        raise ValueError
    return jq_symbol


def gm_symbol_to_ts(gm_symbol: str) -> str:
    """掘金代码转Tushare代码

    :param gm_symbol: 掘金代码
    :return: ts_symbol
    """
    exchange, code = gm_symbol.split(".")
    if exchange == 'SHSE':
        ts_symbol = code + ".SH"
    elif exchange == 'SZSE':
        ts_symbol = code + ".SZ"
    else:
        raise ValueError
    return ts_symbol


def tdx_symbol_to_jq(tdx_symbol):
    """将通达信的代码转成聚宽代码"""
    exchange, code = tdx_symbol[0], tdx_symbol[1:]
    if exchange == '0':
        jq_symbol = code + ".XSHE"
    elif exchange == '1':
        jq_symbol = code + ".XSHG"
    else:
        print(f"{exchange} - {code} is not supported.")
        return None
    return jq_symbol


def jq_symbol_to_tdx(jq_symbol):
    """将聚宽的代码转成通达信代码"""
    code, exchange = jq_symbol.split(".")
    if exchange == 'XSHE':
        tdx_symbol = "0" + code
    elif exchange == 'XSHG':
        tdx_symbol = "1" + code
    else:
        print(f"{exchange} - {code} is not supported.")
        return None
    return tdx_symbol



# coding: utf-8
import zipfile
from tqdm import tqdm
from czsc.analyze import *
from czsc.enum import Freq, Operate
from czsc import signals
from czsc.signals import get_default_signals, get_s_three_bi
from czsc.objects import Event, Factor, Signal

cur_path = os.path.split(os.path.realpath(__file__))[0]


def read_1min():
    with zipfile.ZipFile(os.path.join(cur_path, 'data/000001.XSHG_1min.zip'), 'r') as z:
        f = z.open('000001.XSHG_1min.csv')
        data = pd.read_csv(f, encoding='utf-8')

    data['dt'] = pd.to_datetime(data['dt'])
    records = data.to_dict('records')

    bars = []
    for row in tqdm(records, desc='read_1min'):
        bar = RawBar(**row)
        bar.freq = Freq.F1
        bars.append(bar)
    return bars


def test_find_bi():
    file_kline = os.path.join(cur_path, "data/000001.SH_D.csv")
    kline = pd.read_csv(file_kline, encoding="utf-8")
    kline.loc[:, "dt"] = pd.to_datetime(kline.dt)
    bars = [RawBar(symbol=row['symbol'], id=i, freq=Freq.F1, open=row['open'], dt=row['dt'],
                   close=row['close'], high=row['high'], low=row['low'], vol=row['vol'])
            for i, row in kline.iterrows()]

    # 去除包含关系
    bars1 = []
    for bar in bars:
        if len(bars1) < 2:
            bars1.append(NewBar(symbol=bar.symbol, id=bar.id, freq=bar.freq,
                                dt=bar.dt, open=bar.open,
                                close=bar.close, high=bar.high, low=bar.low,
                                vol=bar.vol, elements=[bar]))
        else:
            k1, k2 = bars1[-2:]
            has_include, k3 = remove_include(k1, k2, bar)
            if has_include:
                bars1[-1] = k3
            else:
                bars1.append(k3)

    fxs = []
    for i in range(1, len(bars1) - 1):
        fx = check_fx(bars1[i - 1], bars1[i], bars1[i + 1])
        if isinstance(fx, FX):
            fxs.append(fx)


def get_user_signals(c: CZSC) -> OrderedDict:
    """在 CZSC 对象上计算信号，这个是标准函数，主要用于研究。
    实盘时可以按照自己的需要自定义计算哪些信号

    :param c: CZSC 对象
    :return: 信号字典
    """
    s = OrderedDict({"symbol": c.symbol, "dt": c.bars_raw[-1].dt, "close": c.bars_raw[-1].close})
    # 倒0，特指未确认完成笔
    # 倒1，倒数第1笔的缩写，表示第N笔
    # 倒2，倒数第2笔的缩写，表示第N-1笔
    # 倒3，倒数第3笔的缩写，表示第N-2笔
    # 以此类推
    for i in range(1, 8):
        s.update(get_s_three_bi(c, i))
    return s


def test_czsc_update():
    file_kline = os.path.join(cur_path, "data/000001.SH_D.csv")
    kline = pd.read_csv(file_kline, encoding="utf-8")
    kline.loc[:, "dt"] = pd.to_datetime(kline.dt)
    bars = [RawBar(symbol=row['symbol'], id=i, freq=Freq.D, open=row['open'], dt=row['dt'],
                   close=row['close'], high=row['high'], low=row['low'], vol=row['vol'])
            for i, row in kline.iterrows()]

    # 不计算任何信号
    c = CZSC(bars, max_bi_count=50)
    assert not c.signals

    # 计算信号
    c = CZSC(bars, max_bi_count=50, get_signals=get_default_signals)
    assert isinstance(c.signals, OrderedDict) and len(c.signals) == 38

    # 测试自定义信号
    c = CZSC(bars, max_bi_count=50, get_signals=get_user_signals)
    assert len(c.signals) == 10

    kline = [x.__dict__ for x in c.bars_raw]
    bi = [{'dt': x.fx_a.dt, "bi": x.fx_a.fx} for x in c.bi_list] + \
         [{'dt': c.bi_list[-1].fx_b.dt, "bi": c.bi_list[-1].fx_b.fx}]
    chart = kline_pro(kline, bi=bi, title="{} - {}".format(c.symbol, c.freq))
    file_html = "x.html"
    chart.render(file_html)
    os.remove(file_html)


def test_czsc_trader():
    bars = read_1min()
    kg = KlineGenerator(max_count=3000, freqs=['1分钟', '5分钟', '15分钟', '30分钟', '60分钟', '日线'])
    for row in tqdm(bars[:-10000], desc='init kg'):
        kg.update(row)

    events = [
        Event(name="开多", operate=Operate.LO, factors=[
            Factor(name="5分钟三买", signals_all=[Signal("5分钟_倒1笔_类买卖点_类三买_任意_任意_0")]),
        ]),

        Event(name="平多", operate=Operate.LE, factors=[
            Factor(name="1分钟一卖", signals_all=[Signal("1分钟_倒1笔_类买卖点_类一卖_任意_任意_0")]),
            Factor(name="5分钟一卖", signals_all=[Signal("5分钟_倒1笔_类买卖点_类一卖_任意_任意_0")]),
            Factor(name="5分钟二卖", signals_all=[Signal("5分钟_倒1笔_类买卖点_类二卖_任意_任意_0")]),
            Factor(name="5分钟三卖", signals_all=[Signal("5分钟_倒1笔_类买卖点_类三卖_任意_任意_0")])
        ]),
    ]
    ct = CzscTrader(op_freq=Freq.F5, kg=kg, get_signals=get_default_signals, events=events)
    assert len(ct.s) == 215
    for row in bars[-10000:]:
        op = ct.check_operate(row)
        print(" : op    : ", op)
        print(" : cache : ", dict(ct.cache), "\n")
    assert len(ct.s) == 215


def test_get_signals():

    def get_test_signals(c: CZSC) -> OrderedDict:
        s = OrderedDict({"symbol": c.symbol, "dt": c.bars_raw[-1].dt, "close": c.bars_raw[-1].close})
        s.update(signals.get_s_d0_bi(c))
        return s

    file_kline = os.path.join(cur_path, "data/000001.SH_D.csv")
    kline = pd.read_csv(file_kline, encoding="utf-8")
    kline.loc[:, "dt"] = pd.to_datetime(kline.dt)
    bars = [RawBar(symbol=row['symbol'], id=i, freq=Freq.D, open=row['open'], dt=row['dt'],
                   close=row['close'], high=row['high'], low=row['low'], vol=row['vol'])
            for i, row in kline.iterrows()]

    # 不计算任何信号
    c = CZSC(bars, max_bi_count=50, get_signals=get_test_signals)
    assert c.signals['日线_倒0笔_方向'] == '向下_任意_任意_0'
    assert c.signals['日线_倒0笔_长度'] == '5到9根K线_任意_任意_0'



# coding: utf-8
import czsc
from os import path
from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="czsc",
    version=czsc.__version__,
    author=czsc.__author__,
    author_email=czsc.__email__,
    keywords=["缠论", "技术分析", "A股", "期货", "缠中说禅"],
    description="缠中说禅技术分析工具",
    long_description=long_description,
    long_description_content_type='text/markdown',
    license="Apache Software License",

    url="https://github.com/zengbin93/czsc",
    packages=find_packages(exclude=['test', 'images', 'docs', 'examples']),
    include_package_data=True,
    install_requires=["pandas", "pyecharts", "tushare", "requests", "seaborn", "transitions"],
    package_data={'': ['data/*.csv']},
    classifiers=[
            'License :: OSI Approved :: Apache Software License',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
        ]
)



# coding: utf-8

from .echarts_plot import kline_pro, heat_map
from .kline_generator import KlineGenerator
from .ta import KDJ, MACD, EMA, SMA
from .io import read_pkl, save_pkl, read_json, save_json
from .log import create_logger



# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2021/8/3 17:10
describe: 掘金量化回测utils
"""
import os

from .base import *


def report_account_status(context, account_id=None):
    """报告账户持仓状态"""
    logger = context.logger
    latest_dt = context.now.strftime(dt_fmt)
    logger.info("=" * 30 + f" 账户状态【{latest_dt}】 " + "=" * 30)

    if context.mode == MODE_BACKTEST:
        account = context.account()
    else:
        account = context.account(account_id=account_id)

    cash = account.cash
    positions = account.positions(symbol="", side="")

    cash_report = f"净值：{int(cash.nav)}，可用资金：{int(cash.available)}，" \
                  f"浮动盈亏：{int(cash.fpnl)}，标的数量：{len(positions)}"
    logger.info(cash_report)

    for p in positions:
        p_report = f"标的：{p.symbol}，名称：{context.shares.get(p.symbol, '无名标的')}，" \
                   f"数量：{p.volume}，成本：{round(p.vwap, 2)}，方向：{p.side}，" \
                   f"当前价：{round(p.price, 2)}，成本市值：{int(p.volume * p.vwap)}，" \
                   f"建仓时间：{p.created_at.strftime(dt_fmt)}"
        logger.info(p_report)


def on_order_status(context, order):
    """
    https://www.myquant.cn/docs/python/python_object_trade#007ae8f5c7ec5298

    :param context:
    :param order:
    :return:
    """
    latest_dt = context.now.strftime("%Y-%m-%d %H:%M:%S")
    logger = context.logger
    symbol = order.symbol
    trader = context.symbols_map[symbol]['trader']
    file_orders = context.file_orders
    msg = f"订单状态更新通知：\n{'*' * 31}\n" \
          f"时间：{latest_dt}\n" \
          f"标的：{order.symbol}\n" \
          f"操作：{trader.op['operate'] + '#' + trader.cache['last_op_desc']}\n" \
          f"方向：{order.side}\n" \
          f"价格：{round(order.price, 2)}\n" \
          f"状态：{order.status}\n" \
          f"委托量：{int(order.volume)}\n" \
          f"已成量：{int(order.filled_volume)}\n" \
          f"均价：{round(order.filled_vwap, 2)}"

    logger.info(msg.replace("\n", " - "))
    if context.mode == MODE_BACKTEST:
        with open(file_orders, 'a', encoding="utf-8") as f:
            f.write(str(order) + '\n')
    else:
        if order.status in [1, 3]:
            push_text(content=str(msg), key=context.wx_key)


def on_execution_report(context, execrpt):
    """响应委托被执行事件，委托成交或者撤单拒绝后被触发。

    https://www.myquant.cn/docs/python/python_trade_event#on_execution_report%20-%20%E5%A7%94%E6%89%98%E6%89%A7%E8%A1%8C%E5%9B%9E%E6%8A%A5%E4%BA%8B%E4%BB%B6
    https://www.myquant.cn/docs/python/python_object_trade#ExecRpt%20-%20%E5%9B%9E%E6%8A%A5%E5%AF%B9%E8%B1%A1

    :param context:
    :param execrpt:
    :return:
    """
    latest_dt = context.now.strftime(dt_fmt)
    logger = context.logger
    msg = f"委托订单被执行通知：\n{'*' * 31}\n" \
          f"时间：{latest_dt}\n" \
          f"标的：{execrpt.symbol}\n" \
          f"方向：{execrpt.side}\n" \
          f"成交量：{int(execrpt.volume)}\n" \
          f"成交价：{round(execrpt.price, 2)}\n" \
          f"执行回报类型：{execrpt.exec_type}"

    logger.info(msg.replace("\n", " - "))
    if context.mode != MODE_BACKTEST and execrpt.exec_type in [1, 5, 6, 8, 12, 19]:
        push_text(content=str(msg), key=context.wx_key)


def on_backtest_finished(context, indicator):
    """

    :param context:
    :param indicator:
        https://www.myquant.cn/docs/python/python_object_trade#bd7f5adf22081af5
    :return:
    """
    wx_key = context.wx_key
    symbols = context.symbols
    data_path = context.data_path
    logger = context.logger

    logger.info(str(indicator))
    logger.info("回测结束 ... ")
    cash = context.account().cash

    for k, v in indicator.items():
        if isinstance(v, float):
            indicator[k] = round(v, 4)

    row = OrderedDict({
        "研究标的": ", ".join(list(context.symbols_map.keys())),
        "回测开始时间": context.backtest_start_time,
        "回测结束时间": context.backtest_end_time,
        "累计收益率": indicator['pnl_ratio'],
        "最大回撤": indicator['max_drawdown'],
        "年化收益率": indicator['pnl_ratio_annual'],
        "夏普比率": indicator['sharp_ratio'],
        "盈利次数": indicator['win_count'],
        "亏损次数": indicator['lose_count'],
        "交易胜率": indicator['win_ratio'],
        "累计出入金": int(cash['cum_inout']),
        "累计交易额": int(cash['cum_trade']),
        "累计手续费": int(cash['cum_commission']),
        "累计平仓收益": int(cash['cum_pnl']),
        "净收益": int(cash['pnl']),
    })
    df = pd.DataFrame([row])
    df.to_excel(os.path.join(context.data_path, "回测结果.xlsx"), index=False)
    logger.info("回测结果：{}".format(row))
    content = ""
    for k, v in row.items():
        content += "{}: {}\n".format(k, v)
    push_text(content=content, key=wx_key)

    trades = []
    for symbol in symbols:
        file_bs = os.path.join(data_path, "cache/{}_bs.txt".format(symbol))
        if os.path.exists(file_bs):
            lines = [eval(x) for x in open(file_bs, 'r', encoding="utf-8").read().strip().split("\n")]
            trades.extend(lines)
    df = pd.DataFrame(trades)
    file_trades = os.path.join(data_path, 'trades.xlsx')
    df.to_excel(file_trades, index=False)
    push_file(file_trades, wx_key)
    push_file(os.path.join(data_path, "backtest.log"), wx_key)
    print(df)


def adjust_future_position_bt(context, symbol: str, trader: CzscTrader, mp: float):
    """调整单只标的仓位

    :param mp: 开仓比例限制
    :param trader:
    :param context:
    :param symbol: 交易标的
    :return:
    """
    assert 1 >= mp >= 0, "开仓限制错误"

    if context.mode == MODE_BACKTEST:
        account = context.account()
    else:
        account = context.account(account_id=context.future_id)

    # 判断是否需要平多仓
    long_position = account.positions(symbol=symbol, side=PositionSide_Long)
    if long_position:
        lp = long_position[0].available
        if lp > 0 and trader.op['operate'] == Operate.LE.value:
            context.logger.info("{} - 平多 - {} - {}".format(symbol, trader.op['desc'], trader.latest_price))
            write_bs(context, symbol, trader.op)
            order_volume(symbol=symbol, volume=lp, side=OrderSide_Sell, order_type=OrderType_Market,
                         position_effect=PositionEffect_Close, account=account.id)
            take_snapshot(context, trader, name=trader.op['desc'])

    # 判断是否需要平空仓
    short_position = account.positions(symbol=symbol, side=PositionSide_Short)
    if short_position:
        sp = short_position[0].available
        if sp > 0 and trader.op['operate'] == Operate.SE.value:
            context.logger.info("{} - 平空 - {} - {}".format(symbol, trader.op['desc'], trader.latest_price))
            write_bs(context, symbol, trader.op)
            order_volume(symbol=symbol, volume=sp, side=OrderSide_Buy, order_type=OrderType_Market,
                         position_effect=PositionEffect_Close, account=account.id)
            take_snapshot(context, trader, name=trader.op['desc'])

    # 判断是否需要开多仓
    if not long_position and trader.op['operate'] == Operate.LO.value:
        context.logger.info("{} - 开多 - {} - {}".format(symbol, trader.op['desc'], trader.latest_price))
        if mp >= 1:
            order_volume(symbol=symbol, volume=mp, side=OrderSide_Buy, order_type=OrderType_Market,
                         position_effect=PositionEffect_Open, account=account.id)
        else:
            order_target_percent(symbol=symbol, percent=mp, position_side=PositionSide_Long,
                                 order_type=OrderType_Market, account=account.id)
        take_snapshot(context, trader, name=trader.op['desc'])
        write_bs(context, symbol, trader.op)

    # 判断是否需要开空仓
    if not short_position and trader.op['operate'] == Operate.SO.value:
        context.logger.info("{} - 开空 - {} - {}".format(symbol, trader.op['desc'], trader.latest_price))
        if mp >= 1:
            order_volume(symbol=symbol, volume=mp, side=OrderSide_Sell, order_type=OrderType_Market,
                         position_effect=PositionEffect_Open, account=account.id)
        else:
            order_target_percent(symbol=symbol, percent=mp, position_side=PositionSide_Short,
                                 order_type=OrderType_Market, account=account.id)
        take_snapshot(context, trader, name=trader.op['desc'])
        write_bs(context, symbol, trader.op)


def adjust_share_position_bt(context, symbol: str, trader: CzscTrader, mp: float, T0=False):
    """调整A股（股票、可转债、ETF）单只标的仓位

    :param T0: 是否允许 T+0 交易，默认为 False，表示不允许 T+0
    :param mp: 开仓比例限制
    :param trader:
    :param context:
    :param symbol: 交易标的
    :return:
    """
    assert 1 >= mp >= 0, "开仓限制错误"
    if symbol[:7] in ['SZSE.12', 'SHSE.11']:
        assert T0, "可转债支持 T+0 操作"

    in_trade_time = "14:59" > context.now.strftime("%H:%M") > "09:31"
    if not in_trade_time:
        return

    if context.mode == MODE_BACKTEST:
        account = context.account()
    else:
        account = context.account(account_id=context.share_id)

    long_position = account.positions(symbol=symbol, side=PositionSide_Long)
    if long_position:
        # 判断是否需要平多仓
        if T0:
            lp = long_position[0].volume
        else:
            lp = long_position[0].volume - long_position[0].volume_today

        if trader.op['operate'] in [Operate.LE.value, Operate.HO.value]:
            trader.op['desc'] = trader.cache['last_op_desc']
            if lp <= 0:
                context.logger.info("{} - 可平仓位为零 - {} - {}".format(symbol, trader.op['desc'], trader.latest_price))
            else:
                context.logger.info("{} - 平多 - {} - {}".format(symbol, trader.op['desc'], trader.latest_price))
                order_volume(symbol=symbol, volume=lp, side=OrderSide_Sell, order_type=OrderType_Market,
                             position_effect=PositionEffect_Close, account=account.id)
                take_snapshot(context, trader, name=trader.op['desc'])
                write_bs(context, symbol, trader.op)
    else:
        # 判断是否需要开多仓
        if trader.op['operate'] in [Operate.LO.value, Operate.HL.value]:
            trader.op['desc'] = trader.cache['last_op_desc']
            context.logger.info("{} - 开多 - {} - {}".format(symbol, trader.op['desc'], trader.latest_price))

            if trader.kg.m1[-1].id - trader.cache['long_open_k1_id'] < context.wait_time:
                context.logger.info(f"{symbol}，开仓条件满足，等待开仓中，"
                                    f"已等待 {trader.kg.m1[-1].id - trader.cache['long_open_k1_id']}")
                return

            # 判断当前价格是否在开仓容差范围
            if abs(trader.latest_price - trader.cache['long_open_price']) > trader.cache['long_open_price'] * 0.03:
                context.logger.info(f"{symbol}，当前价（{trader.latest_price}）不在"
                                    f"开仓价（{trader.cache['long_open_price']}）容差范围，不允许开仓")
                return

            # 判断总仓位是否已经达到限制水平
            assert account.cash.available > account.cash.order_frozen
            cash_left = account.cash.available - account.cash.order_frozen
            if 1 - cash_left / account.cash.nav > context.max_total_position:
                context.logger.info(f"当前持仓比例已经超过{context.max_total_position * 100}%，不允许再开仓")
                return

            if account.cash.available < max(105 * trader.latest_price, 20000):
                context.logger.info(
                    "{} - 开多信号触发 - {} - {}；可用资金小于2万 / 可用仓位不够开1手，暂不开仓".format(
                        symbol, trader.op['desc'], trader.latest_price))
                return

            if mp >= 1:
                max_mp = account.cash.available // (100 * trader.latest_price)
                mp = min(mp, max_mp)
                order_volume(symbol=symbol, volume=mp * 100, side=OrderSide_Buy, order_type=OrderType_Limit,
                             price=trader.latest_price, position_effect=PositionEffect_Open, account=account.id)
            else:
                max_mp = (account.cash.available * 0.95) / account.cash.nav
                mp = min(max_mp, mp)
                # 实盘陷阱：市价柜台是按照涨停价冻结资金的
                # 修改成 按最近一个1分钟bar的close开仓
                order_target_percent(symbol=symbol, percent=mp, position_side=PositionSide_Long,
                                     order_type=OrderType_Limit, price=trader.latest_price, account=account.id)
            take_snapshot(context, trader, name=trader.op['desc'])
            write_bs(context, symbol, trader.op)


def adjust_position_bt(context, symbol):
    bars = context.data(symbol=symbol, frequency='60s', count=100, fields='symbol,eob,open,close,high,low,volume')
    trader: CzscTrader = context.symbols_map[symbol]['trader']
    mp = context.symbols_map[symbol]['mp']
    bars = format_kline(bars, freq=Freq.F1)
    bars_new = [x for x in bars if x.dt > trader.kg.end_dt]
    if bars_new:
        for k in bars_new:
            trader.check_operate(k, context.stoploss, context.timeout, context.wait_time, context.max_open_tolerance)
    context.symbols_map[symbol]['trader'] = trader
    context.logger.info(context.shares.get(symbol, "无名标的") + " : op    : " + str(trader.op))
    context.logger.info(context.shares.get(symbol, "无名标的") + " : cache : " + str(dict(trader.cache)))

    # 可转债支持 T+0 交易
    if symbol[:7] in ['SZSE.12', 'SHSE.11']:
        adjust_share_position_bt(context, symbol, trader, mp, T0=True)
        return

    exchange, sec_code = symbol.split(".")
    if exchange in ["SZSE", "SHSE"]:
        adjust_share_position_bt(context, symbol, trader, mp)
    else:
        adjust_future_position_bt(context, symbol, trader, mp)


def on_bar(context, bars):
    context.unfinished_orders = get_unfinished_orders()

    for bar in bars:
        symbol = bar['symbol']
        try:
            adjust_position_bt(context, symbol)
        except:
            traceback.print_exc()

    if context.now.hour == 13 and context.now.minute == 59:
        report_account_status(context)


def init_context_bt(context, name, symbols,
                    op_freq: Freq,
                    freqs: List[str],
                    get_signals: Callable,
                    get_events: Callable):
    """回测 context 初始化方法

    :param context: 掘金context
    :param name: tactic 的名称
    :param symbols: 交易标的列表
    :param op_freq: 指定的操作级别
    :param freqs: K线周期列表
    :param get_signals: 信号计算方法
    :param get_events: 事件列表获取
    :return:
    """
    path_gm_logs = os.environ.get('path_gm_logs', None)
    assert os.path.exists(path_gm_logs)
    data_path = os.path.join(path_gm_logs, f"backtest/{name}_{datetime.now().strftime('%Y%m%d_%H%M%S')}")
    context.wx_key = os.environ['wx_key']
    cache_path = os.path.join(data_path, "cache")
    os.makedirs(cache_path, exist_ok=False)
    context.logger = create_logger(os.path.join(data_path, "backtest.log"), cmd=True, name="gm")
    context.data_path = data_path
    context.cache_path = cache_path
    context.file_orders = os.path.join(data_path, "orders.txt")
    context.shares = get_shares()
    context.stoploss = float(os.environ['stoploss'])  # 止损条件设定
    context.timeout = int(os.environ['timeout'])      # 超时条件设定
    context.wait_time = int(os.environ['wait_time'])  # 开仓等待时长，单位：分钟
    context.max_open_tolerance = float(os.environ['max_open_tolerance'])    # 最大开仓容错百分比

    # 仓位控制[0, 1]，按资金百分比控制，1表示满仓，仅在开仓的时候控制
    context.max_total_position = float(os.environ['max_total_position'])
    context.max_share_position = float(os.environ['max_share_position'])
    assert 0 <= context.max_total_position <= 1
    assert 0 <= context.max_share_position <= 1
    assert context.max_total_position >= context.max_share_position >= 0

    context.logger.info("回测配置：")
    context.logger.info("backtest_start_time = " + str(context.backtest_start_time))
    context.logger.info("backtest_end_time = " + str(context.backtest_end_time))
    context.logger.info(f"总仓位控制：{context.max_total_position}")
    context.logger.info(f"单仓位控制：{context.max_share_position}")
    context.logger.info(f"异常退出条件：stoploss = {context.stoploss}; timeout = {context.timeout}; wait_time = {context.wait_time}")
    context.logger.info(f"K线周期列表：{freqs}")
    context.logger.info("交易信号计算：\n" + inspect.getsource(get_signals))
    context.logger.info("交易事件定义：\n" + inspect.getsource(get_events))

    events = get_events()
    context.logger.info(f"交易标的数量：{len(symbols)}")
    symbols_map = {symbol: dict() for symbol in symbols}
    for symbol in symbols:
        try:
            symbols_map[symbol]['mp'] = context.max_share_position
            kg = get_init_kg(symbol, context.now, max_count=2000, adjust=ADJUST_POST, freqs=freqs)
            trader = CzscTrader(op_freq=op_freq, kg=kg, get_signals=get_signals, events=events)
            symbols_map[symbol]['trader'] = trader
            context.logger.info("{} 初始化完成，当前时间：{}".format(symbol, trader.end_dt))
        except:
            del symbols_map[symbol]
            print(f'init kg fail on {symbol}')

    subscribe(",".join(symbols_map.keys()), frequency='60s', count=300, wait_group=True)
    context.logger.info(f"交易标的配置：{symbols_map}")
    context.symbols_map = symbols_map




# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2021/8/3 17:25
"""
import traceback
from czsc.objects import Signal, Factor, Event
from czsc.utils.io import read_pkl, save_pkl

from scripts_gm.src.utils.base import *


def get_monitor_signals(c):
    s = get_default_signals(c)
    return s


def stocks_monitor(symbols, wx_key):
    """对实盘股票池进行定时监控"""
    events = [
        # 开多
        Event(name="一买", operate=Operate.LO, factors=[
            Factor(name="5分钟类一买", signals_all=[Signal("5分钟_倒1笔_类买卖点_类一买_任意_任意_0")]),
            Factor(name="5分钟形一买", signals_all=[Signal("5分钟_倒1笔_基础形态_类一买_任意_任意_0")]),

            Factor(name="15分钟类一买", signals_all=[Signal("15分钟_倒1笔_类买卖点_类一买_任意_任意_0")]),
            Factor(name="15分钟形一买", signals_all=[Signal("15分钟_倒1笔_基础形态_类一买_任意_任意_0")]),

            Factor(name="30分钟类一买", signals_all=[Signal("30分钟_倒1笔_类买卖点_类一买_任意_任意_0")]),
            Factor(name="30分钟形一买", signals_all=[Signal("30分钟_倒1笔_基础形态_类一买_任意_任意_0")]),
        ]),

        Event(name="二买", operate=Operate.LO, factors=[
            Factor(name="5分钟类二买", signals_all=[Signal("5分钟_倒1笔_类买卖点_类二买_任意_任意_0")]),
            Factor(name="5分钟形二买", signals_all=[Signal("5分钟_倒1笔_基础形态_类二买_任意_任意_0")]),

            Factor(name="15分钟类二买", signals_all=[Signal("15分钟_倒1笔_类买卖点_类二买_任意_任意_0")]),
            Factor(name="15分钟形二买", signals_all=[Signal("15分钟_倒1笔_基础形态_类二买_任意_任意_0")]),

            Factor(name="30分钟类二买", signals_all=[Signal("30分钟_倒1笔_类买卖点_类二买_任意_任意_0")]),
            Factor(name="30分钟形二买", signals_all=[Signal("30分钟_倒1笔_基础形态_类二买_任意_任意_0")]),
        ]),
        Event(name="三买", operate=Operate.LO, factors=[
            Factor(name="5分钟类三买", signals_all=[Signal("5分钟_倒1笔_类买卖点_类三买_任意_任意_0")]),
            Factor(name="5分钟形三买", signals_all=[Signal("5分钟_倒1笔_基础形态_类三买_任意_任意_0")]),

            Factor(name="15分钟类三买", signals_all=[Signal("15分钟_倒1笔_类买卖点_类三买_任意_任意_0")]),
            Factor(name="15分钟形三买", signals_all=[Signal("15分钟_倒1笔_基础形态_类三买_任意_任意_0")]),

            Factor(name="30分钟类三买", signals_all=[Signal("30分钟_倒1笔_类买卖点_类三买_任意_任意_0")]),
            Factor(name="30分钟形三买", signals_all=[Signal("30分钟_倒1笔_基础形态_类三买_任意_任意_0")]),
        ]),

        # 平多
        Event(name="一卖", operate=Operate.LE, factors=[
            Factor(name="5分钟类一卖", signals_all=[Signal("5分钟_倒1笔_类买卖点_类一卖_任意_任意_0")]),
            Factor(name="5分钟形一卖", signals_all=[Signal("5分钟_倒1笔_基础形态_类一卖_任意_任意_0")]),

            Factor(name="15分钟类一卖", signals_all=[Signal("15分钟_倒1笔_类买卖点_类一卖_任意_任意_0")]),
            Factor(name="15分钟形一卖", signals_all=[Signal("15分钟_倒1笔_基础形态_类一卖_任意_任意_0")]),

            Factor(name="30分钟类一卖", signals_all=[Signal("30分钟_倒1笔_类买卖点_类一卖_任意_任意_0")]),
            Factor(name="30分钟形一卖", signals_all=[Signal("30分钟_倒1笔_基础形态_类一卖_任意_任意_0")]),
        ]),
        Event(name="二卖", operate=Operate.LE, factors=[
            Factor(name="5分钟类二卖", signals_all=[Signal("5分钟_倒1笔_类买卖点_类二卖_任意_任意_0")]),
            Factor(name="5分钟形二卖", signals_all=[Signal("5分钟_倒1笔_基础形态_类二卖_任意_任意_0")]),

            Factor(name="15分钟类二卖", signals_all=[Signal("15分钟_倒1笔_类买卖点_类二卖_任意_任意_0")]),
            Factor(name="15分钟形二卖", signals_all=[Signal("15分钟_倒1笔_基础形态_类二卖_任意_任意_0")]),

            Factor(name="30分钟类二卖", signals_all=[Signal("30分钟_倒1笔_类买卖点_类二卖_任意_任意_0")]),
            Factor(name="30分钟形二卖", signals_all=[Signal("30分钟_倒1笔_基础形态_类二卖_任意_任意_0")]),
        ]),
        Event(name="三卖", operate=Operate.LE, factors=[
            Factor(name="5分钟类三卖", signals_all=[Signal("5分钟_倒1笔_类买卖点_类三卖_任意_任意_0")]),
            Factor(name="5分钟形三卖", signals_all=[Signal("5分钟_倒1笔_基础形态_类三卖_任意_任意_0")]),

            Factor(name="15分钟类三卖", signals_all=[Signal("15分钟_倒1笔_类买卖点_类三卖_任意_任意_0")]),
            Factor(name="15分钟形三卖", signals_all=[Signal("15分钟_倒1笔_基础形态_类三卖_任意_任意_0")]),

            Factor(name="30分钟类三卖", signals_all=[Signal("30分钟_倒1笔_类买卖点_类三卖_任意_任意_0")]),
            Factor(name="30分钟形三卖", signals_all=[Signal("30分钟_倒1笔_基础形态_类三卖_任意_任意_0")]),
        ]),
    ]

    shares = get_shares()
    data_path = './data/monitor'
    os.makedirs(data_path, exist_ok=True)
    push_text("实盘池监控启动 @ {}".format(datetime.now().strftime("%Y-%m-%d %H:%M")), wx_key)

    for symbol in symbols:
        try:
            file_ct = os.path.join(data_path, "{}.ct".format(symbol))
            if os.path.exists(file_ct):
                ct: GmCzscTrader = read_pkl(file_ct)
                ct.update_factors()
            else:
                ct = GmCzscTrader(symbol, max_count=2000, get_signals=get_monitor_signals)
            save_pkl(ct, file_ct)
            print(f"run monitor on {symbol} at {ct.end_dt}")
            msg = f"标的代码：{symbol}\n标的名称：{shares.get(symbol, '')}\n" \
                  f"同花顺F10：http://basic.10jqka.com.cn/{symbol.split('.')[1]}\n"
            msg += f"新浪行情：https://finance.sina.com.cn/realstock/company/{symbol[:2].lower()}{symbol[-6:]}/nc.shtml\n"
            for event in events:
                m, f = event.is_match(ct.s)
                if m:
                    msg += "监控提醒：{}@{}\n".format(event.name, f)

            if "监控提醒" in msg:
                push_text(msg.strip("\n"), key=wx_key)

        except Exception as e:
            traceback.print_exc()
            print("{} 执行失败 - {}".format(symbol, e))
    push_text("实盘池监控结束 @ {}".format(datetime.now().strftime("%Y-%m-%d %H:%M")), wx_key)


def stocks_monitor_rt(context):
    if context.now.isoweekday() >= 6:
        return
    stocks_monitor(list(context.symbols_map.keys()), context.wx_key)


if __name__ == '__main__':
    symbols = pd.read_excel(r"C:\gm_logs\ZB20210726.xlsx")['股票代码'].unique().tolist()
    to_gm_symbol = lambda x: "SHSE." + x[:6] if x[0] == "6" else "SZSE." + x[:6]
    symbols = [to_gm_symbol(x) for x in symbols]
    wx_key = '909731bd-****-46ad-****-24b9830873a4'
    stocks_monitor(symbols, wx_key)




# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2021/7/16 11:51
"""

import os

home_path = os.path.join(os.path.expanduser("~"), '.czsc')
os.makedirs(home_path, exist_ok=True)


